#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "dpu2program4xi.py">
#<abstract4xi abstract = "DPU program management"/>
#<refer4xi reference = "program4xi.py"/>
#<verify4xi verification = "10" target = "dpu2program4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'DPU2Program4Xi',                      # ACK 5.6     p132  

            'add2programwithbinary4dpu',           # ACK 5.6.1   p132            
            'add2programwithsource4dpu',           # ACK 5.6.1   p133  
                                
            'build2program4dpu',                   # ACK 5.6.2   p137      

            'compile2program4dpu',                 # ACK 5.6.3   p139            
            'create2programwithsource4dpu',        # ACK 5.6.1   p132 
            'create2programwithbinary4dpu',        # ACK 5.6.1   p133 
            'create2programwithbuiltinkernels4dpu',# NACK 5.6.1  p135   
                       
            'delete2programwithbinary4dpu',        # ACK  
            'delete2programwithsource4dpu',        # ACK               
            
            'fetch2programbinary4dpu',             # ACK
            'fetch2programsource4dpu',             # ACK
            
            'get2programinfo4dpu',                 # ACK 5.6.7   p150 
            'get2programbuildinfo4dpu',            # NACK 5.6.7  p153
            
            'link2program4dpu',                    # ACK 5.6.3   p142                          
            'release2program4dpu',                 # ACK 5.6.1   p136                                         
            'retain2program4dpu',                  # ACK 5.6.1   p136
 
            'set2programwithsource4dpu',           # ACK
            'set2programwithbinary4dpu',           # ACK            
            'set2programcallback4dpu',             # ACK 
                              
            'trace2program4dpu',                   # ACK
            'unload2compiler4dpu'                  # NACK 5.6.6  p149 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi
from version4xi import VersionKB4Xi

# -------------------------------------------------------------------------------------------------
# Buffer                           
# -------------------------------------------------------------------------------------------------
from dpu2buffer4xi import get2bufferinfo4dpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from dpu2commandqueue4xi import DPU2CommandQueue4Xi
from dpu2commandqueue4xi import create2commandqueue4dpu 
from dpu2commandqueue4xi import flush2commandqueue4dpu
from dpu2commandqueue4xi import finish2commandqueue4dpu
from dpu2commandqueue4xi import get2commandqueueinfo4dpu
from dpu2commandqueue4xi import release2commandqueue4dpu
from dpu2commandqueue4xi import retain2commandqueue4dpu
from dpu2commandqueue4xi import set2commandqueue4dpu
from dpu2commandqueue4xi import trace2commandqueue4dpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from dpu2context4xi import DPU2Context4Xi 
from dpu2context4xi import create2context4dpu
from dpu2context4xi import get2context4dpu
from dpu2context4xi import release2context4dpu
from dpu2context4xi import retain2context4dpu
from dpu2context4xi import set2context4dpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from dpu2device4xi import DPU4Xi
from dpu2device4xi import init2dpu4xi 
from dpu2device4xi import set2dpu4xi
from dpu2device4xi import get2dpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from dpu2event4xi import DPU2Event4Xi
from dpu2event4xi import create2event4dpu
from dpu2event4xi import get2eventinfo4dpu
from dpu2event4xi import release2event4dpu
from dpu2event4xi import retain2event4dpu
from dpu2event4xi import set2eventcallback4dpu 
from dpu2event4xi import set2event4dpu
from dpu2event4xi import trace2event4dpu
from dpu2event4xi import wait2event4dpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from process4xi import Process4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                         
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global dpu2program4xi                                                   # Program of DPU
global dpu2build4xi                                                     # Build of DPU
global dpu2compile4xi                                                   # Compile of DPU
global dpu2trace4xi                                                     # TRACE

global dir2binary4dpu                                                   # Directory for binary files
global dir2source4dpu                                                   # Directory for source files

global fetch2code4source                                                # Fetched source code
global fetch2code4binary                                                # Fetched binary code

global import2use4dpu                                                   # Imports of target function for Process4Xi

global kb2fetch4dpu                                                     # Knowledgebase for fetching

global folder2lib4dpu                                                   # Folder of DPU library

global profile2fetch4source                                             # Extension for source files
global keys2profile4source                                                
global profile2fetch4binary                                             # Extension for binary files
global keys2profile4binary                                               

global target2function4dpu                                              # Target function for Process4Xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
temp2dpu = 'dpu'
key2dir = temp2dpu[0]                                                                                               # MORE COMPLIATED WAY DUE TO AUTOMATIC SOFTWARE GENERATION
folder2lib4dpu = VersionKB4Xi[key2dir] 
dir2binary4dpu = folder2lib4dpu + '__pycache__//'                                                                   # Default directory is Python cache for binary files - C++ exe files are in //bin
dir2source4dpu = folder2lib4dpu                                                                                     # Directory for source files

profile2fetch4source = {}
profile2fetch4source ['.py']  = 'ACK'                                                                               # Example extensions: '.txt'= text '.py'=Python, '.cpp'=C++, '.hpp'=headers, '.c'=C
profile2fetch4source ['.cpp'] = 'ACK'
profile2fetch4source ['.c']   = 'ACK'
profile2fetch4source ['.cl']  = 'ACK'
keys2profile4source = profile2fetch4source.keys()

profile2fetch4binary = {}
profile2fetch4binary ['.pyc'] = 'ACK'                                                                              
profile2fetch4binary ['.exe'] = 'ACK' 
keys2profile4binary = profile2fetch4binary.keys()

kb2fetch4dpu = {}                                                                                                   # Program to be fetched - always define both source and binary pairs                               
kb2fetch4dpu[1] = folder2lib4dpu + 'dpu2sampler4xi5test.py'                                                         # This will be fetched if no parameters are given to create function
kb2fetch4dpu[2] = folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'
kb2fetch4dpu[3] = folder2lib4dpu + 'dpu2dummy4xi5test.cpp'                                                          # C++ 
kb2fetch4dpu[4] = folder2lib4dpu + 'bin//dpu2dummy4xi5test.exe'



# kb2fetch4dpu[1]  = folder2lib4dpu + 'dpu2sampler4xi5test.py'                                                      
# kb2fetch4dpu[2]  = folder2lib4dpu + '__pycache__//dpu2sampler4xi.cpython-33.pyc'                                  

# kb2fetch4dpu[3]  = folder2lib4dpu + 'dpu2image4xi5test.py'                                                        # Note for an unresolved reason Python compiler does NOT always generate cache file!
# kb2fetch4dpu[4]  = folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'
# kb2fetch4dpu[5]  = folder2lib4dpu + 'dpu3image4xi5test.py'
# kb2fetch4dpu[6]  = folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'
# kb2fetch4dpu[7]  = folder2lib4dpu + 'dpu2commandqueue4xi5test.py'
# kb2fetch4dpu[8]  = folder2lib4dpu + '__pycache__//dpu2commandqueue4xi.cpython-33.pyc'
# kb2fetch4dpu[9]  = folder2lib4dpu + 'dpu2event4xi5test.py'
# kb2fetch4dpu[10] = folder2lib4dpu + '__pycache__//dpu2event4xi.cpython-33.pyc'
# kb2fetch4dpu[11]  = folder2lib4dpu + 'dpu2kernel4xi5test.py'
# kb2fetch4dpu[12] = folder2lib4dpu + '__pycache__//dpu2kernel4xi.cpython-33.pyc'
# kb2fetch4dpu[13]  = folder2lib4dpu + 'dpu2dummy4xi5test.cpp'                                                      # C++ 
# kb2fetch4dpu[14] = folder2lib4dpu + 'bin//dpu2dummy4xi5test.exe'

fetch2code4source = {} 
fetch2code4binary = {} 

#"/>

# -------------------------------------------------------------------------------------------------
# Command queue module of DPU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importcreate2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import create2commandqueue4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importflush2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importflush2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import flush2commandqueue4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importfinish2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importfinish2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import finish2commandqueue4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2commandqueueinfo4dpu" usage = " importing target function for Process4Xi">
def importget2commandqueueinfo4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import get2commandqueueinfo4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importrelease2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importrelease2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import release2commandqueue4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importretain2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importretain2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import retain2commandqueue4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importset2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importset2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import set2commandqueue4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importtrace2commandqueue4dpu" usage = " importing target function for Process4Xi">
def importtrace2commandqueue4dpu():
   global target2function4dpu
   from dpu2commandqueue4xi import trace2commandqueue4dpu as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Context module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2context4dpu" usage = " importing target function for Process4Xi">
def importcreate2context4dpu():
   global target2function4dpu
   from dpu2context4xi import create2context4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2context4dpu" usage = " importing target function for Process4Xi">
def importget2context4dpu():
   global target2function4dpu
   from dpu2context4xi import get2context4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importrelease2context4dpu" usage = " importing target function for Process4Xi">
def importrelease2context4dpu():
   global target2function4dpu
   from dpu2context4xi import release2context4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importretain2context4dpu" usage = " importing target function for Process4Xi">
def importretain2context4dpu():
   global target2function4dpu
   from dpu2context4xi import retain2context4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importset2context4dpu" usage = " importing target function for Process4Xi">
def importset2context4dpu():
   global target2function4dpu
   from dpu2context4xi import set2context4dpu as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Device module of DPU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importinit2dpu4xi" usage = " importing target function for Process4Xi">
def importinit2dpu4xi():
   global target2function4dpu
   from dpu2device4xi import init2dpu4xi as target2function4dpu 
#</function4xi> 

#<function4xi function = "importset2dpu4xi" usage = " importing target function for Process4Xi">
def importset2dpu4xi():
   global target2function4dpu
   from dpu2device4xi import set2dpu4xi   as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2dpu4xi" usage = " importing target function for Process4Xi">
def importget2dpu4xi():
   global target2function4dpu
   from dpu2device4xi import get2dpu4xi   as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Event module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2event4dpu" usage = " importing target function for Process4Xi">
def importcreate2event4dpu():
   global target2function4dpu
   from dpu2event4xi import create2event4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2eventinfo4dpu" usage = " importing target function for Process4Xi">
def importget2eventinfo4dpu():
   global target2function4dpu
   from dpu2event4xi import get2eventinfo4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importrelease2event4dpu" usage = " importing target function for Process4Xi">
def importrelease2event4dpu():
   global target2function4dpu
   from dpu2event4xi import release2event4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importretain2event4dpu" usage = " importing target function for Process4Xi">
def importretain2event4dpu():
   global target2function4dpu
   from dpu2event4xi import retain2event4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importset2eventcallback4dpu" usage = " importing target function for Process4Xi">
def importset2eventcallback4dpu():
   global target2function4dpu
   from dpu2event4xi import set2eventcallback4dpu as target2function4dpu 
#</function4xi> 

#<function4xi function = "importset2event4dpu" usage = " importing target function for Process4Xi">
def importset2event4dpu():
   global target2function4dpu
   from dpu2event4xi import set2event4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importtrace2event4dpu" usage = " importing target function for Process4Xi">
def importtrace2event4dpu():
   global target2function4dpu
   from dpu2event4xi import trace2event4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importwait2event4dpu" usage = " importing target function for Process4Xi">
def importwait2event4dpu():
   global target2function4dpu
   from dpu2event4xi import wait2event4dpu as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 2D Image module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2image4dpu" usage = " importing target function for Process4Xi">
def importcreate2image4dpu():
   global target2function4dpu
   from dpu2image4xi import create2image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2imageinfo4dpu" usage = " importing target function for Process4Xi">
def importget2imageinfo4dpu():
   global target2function4dpu
   from dpu2image4xi import get2imageinfo4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2supportedimageformats4dpu" usage = " importing target function for Process4Xi">
def importget2supportedimageformats4dpu():
   global target2function4dpu
   from dpu2image4xi import get2supportedimageformats4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importmanoeuvre2copyimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimage4dpu():
   global target2function4dpu
   from dpu2image4xi import manoeuvre2copyimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copyimagetobuffer4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimagetobuffer4dpu():
   global target2function4dpu
   from dpu2image4xi import manoeuvre2copyimagetobuffer4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copybuffertoimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copybuffertoimage4dpu():
   global target2function4dpu
   from dpu2image4xi import manoeuvre2copybuffertoimage4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importmanoeuvre2readimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2readimage4dpu():
   global target2function4dpu
   from dpu2image4xi import manoeuvre2readimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2writeimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2writeimage4dpu():
   global target2function4dpu
   from dpu2image4xi import manoeuvre2writeimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2mapimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2mapimage4dpu():
   global target2function4dpu
   from dpu2image4xi import manoeuvre2mapimage4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importcopy2image4dpu" usage = " importing target function for Process4Xi">
def importcopy2image4dpu():
   global target2function4dpu
   from dpu2image4xi import copy2image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcopy2imagetobuffer4dpu" usage = " importing target function for Process4Xi">
def importcopy2imagetobuffer4dpu():
   global target2function4dpu
   from dpu2image4xi import copy2imagetobuffer4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcopy2buffertoimage4dpu" usage = " importing target function for Process4Xi">
def importcopy2buffertoimage4dpu():
   global target2function4dpu
   from dpu2image4xi import copy2buffertoimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importread2image4dpu" usage = " importing target function for Process4Xi">
def importread2image4dpu():
   global target2function4dpu
   from dpu2image4xi import read2image4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importmap2image4dpu" usage = " importing target function for Process4Xi">
def importmap2image4dpu():
   global target2function4dpu
   from dpu2image4xi import map2image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importrelease2image4dpu" usage = " importing target function for Process4Xi">
def importrelease2image4dpu():
   global target2function4dpu
   from dpu2image4xi import release2image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importretain2image4dpu" usage = " importing target function for Process4Xi">
def importretain2image4dpu():
   global target2function4dpu
   from dpu2image4xi import retain2image4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importset2imagecallback4dpu" usage = " importing target function for Process4Xi">
def importset2imagecallback4dpu():
   global target2function4dpu
   from dpu2image4xi import set2imagecallback4dpu as target2function4dpu 
#</function4xi>
 
#<function4xi function = "importset2image4dpu" usage = " importing target function for Process4Xi">
def importset2image4dpu():
   global target2function4dpu
   from dpu2image4xi import set2image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importtrace2image4dpu" usage = " importing target function for Process4Xi">
def importtrace2image4dpu():
   global target2function4dpu
   from dpu2image4xi import trace2image4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importwait2image4dpu" usage = " importing target function for Process4Xi">
def importwait2image4dpu():
   global target2function4dpu
   from dpu2image4xi import wait2image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importwrite2image4dpu" usage = " importing target function for Process4Xi">
def importwrite2image4dpu():
   global target2function4dpu
   from dpu2image4xi import write2image4dpu as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 3D Image module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate3image4dpu" usage = " importing target function for Process4Xi">
def importcreate3image4dpu():
   global target2function4dpu
   from dpu3image4xi import create3image4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importget3imageinfo4dpu" usage = " importing target function for Process4Xi">
def importget3imageinfo4dpu():
   global target2function4dpu
   from dpu3image4xi import get3imageinfo4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget3supportedimageformats4dpu" usage = " importing target function for Process4Xi">
def importget3supportedimageformats4dpu():
   global target2function4dpu
   from dpu3image4xi import get3supportedimageformats4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimage4dpu():
   global target2function4dpu
   from dpu3image4xi import manoeuvre3copyimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimagetobuffer4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimagetobuffer4dpu():
   global target2function4dpu
   from dpu3image4xi import manoeuvre3copyimagetobuffer4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importmanoeuvre3copybuffertoimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copybuffertoimage4dpu():
   global target2function4dpu
   from dpu3image4xi import manoeuvre3copybuffertoimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3readimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3readimage4dpu():
   global target2function4dpu
   from dpu3image4xi import manoeuvre3readimage4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importmanoeuvre3writeimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3writeimage4dpu():
   global target2function4dpu
   from dpu3image4xi import manoeuvre3writeimage4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importmanoeuvre3mapimage4dpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3mapimage4dpu():
   global target2function4dpu
   from dpu3image4xi import manoeuvre3mapimage4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcopy3image4dpu" usage = " importing target function for Process4Xi">
def importcopy3image4dpu():
   global target2function4dpu
   from dpu3image4xi import copy3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcopy3imagetobuffer4dpu" usage = " importing target function for Process4Xi">
def importcopy3imagetobuffer4dpu():
   global target2function4dpu
   from dpu3image4xi import copy3imagetobuffer4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcopy3buffertoimage4dpu" usage = " importing target function for Process4Xi">
def importcopy3buffertoimage4dpu():
   global target2function4dpu
   from dpu3image4xi import copy3buffertoimage4dpu as target2function4dpu
#</function4xi>

#<function4xi function = "importread3image4dpu" usage = " importing target function for Process4Xi">
def importread3image4dpu():
   global target2function4dpu
   from dpu3image4xi import read3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importmap3image4dpu" usage = " importing target function for Process4Xi">
def importmap3image4dpu():
   global target2function4dpu
   from dpu3image4xi import map3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importrelease3image4dpu" usage = " importing target function for Process4Xi">
def importrelease3image4dpu():
   global target2function4dpu
   from dpu3image4xi import release3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importretain3image4dpu" usage = " importing target function for Process4Xi">
def importretain3image4dpu():
   global target2function4dpu
   from dpu3image4xi import retain3image4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importset3imagecallback4dpu" usage = " importing target function for Process4Xi">
def importset3imagecallback4dpu():
   global target2function4dpu
   from dpu3image4xi import set3imagecallback4dpu as target2function4dpu 
#</function4xi> 

#<function4xi function = "importset3image4dpu" usage = " importing target function for Process4Xi">
def importset3image4dpu():
   global target2function4dpu
   from dpu3image4xi import set3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importtrace3image4dpu" usage = " importing target function for Process4Xi">
def importtrace3image4dpu():
   global target2function4dpu
   from dpu3image4xi import trace3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importwait3image4dpu" usage = " importing target function for Process4Xi">
def importwait3image4dpu():
   global target2function4dpu
   from dpu3image4xi import wait3image4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importwrite3image4dpu" usage = " importing target function for Process4Xi">
def importwrite3image4dpu():
   global target2function4dpu
   from dpu3image4xi import write3image4dpu as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Sampler module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2sampler4dpu" usage = " importing target function for Process4Xi">
def importcreate2sampler4dpu():
   global target2function4dpu
   from dpu2sampler4xi import create2sampler4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importget2samplerinfo4dpu" usage = " importing target function for Process4Xi">
def importget2samplerinfo4dpu():
   global target2function4dpu
   from dpu2sampler4xi import get2samplerinfo4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importrelease2sampler4dpu" usage = " importing target function for Process4Xi">
def importrelease2sampler4dpu():
   global target2function4dpu
   from dpu2sampler4xi import release2sampler4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importretain2sampler4dpu" usage = " importing target function for Process4Xi">
def importretain2sampler4dpu():
   global target2function4dpu
   from dpu2sampler4xi import retain2sampler4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importset2samplercallback4dpu" usage = " importing target function for Process4Xi">
def importset2samplercallback4dpu():
   global target2function4dpu
   from dpu2sampler4xi import set2samplercallback4dpu as target2function4dpu 
#</function4xi>
 
#<function4xi function = "importset2sampler4dpu" usage = " importing target function for Process4Xi">
def importset2sampler4dpu():
   global target2function4dpu
   from dpu2sampler4xi import set2sampler4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importtrace2sampler4dpu" usage = " importing target function for Process4Xi">
def importtrace2sampler4dpu():
   global target2function4dpu
   from dpu2sampler4xi import trace2sampler4dpu as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Kernel module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importadd2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importadd2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import add2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi>

#<function4xi function = "importexecute2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importexecute2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import execute2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importexecute2task4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importexecute2task4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import execute2task4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importexecute2nativekernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importexecute2nativekernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import execute2nativekernel4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcreate2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importcreate2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import create2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importcreate2kernelsinprogram4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importcreate2kernelsinprogram4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import create2kernelsinprogram4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importdelete2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importdelete2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import delete2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importfetch2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importfetch2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import fetch2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importget2kernelinfo4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importget2kernelinfo4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import get2kernelinfo4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2kernelworkgroupinfo4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importget2kernelworkgroupinfo4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import get2kernelworkgroupinfo4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importget2kernelarginfo4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importget2kernelarginfo4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import get2kernelarginfo4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importrelease2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importrelease2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import release2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi> 

#<function4xi function = "importretain2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importretain2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import retain2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importset2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importset2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import set2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importset2kernelcallback4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importset2kernelcallback4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import set2kernelcallback4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importset2kernelarg4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importset2kernelarg4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import set2kernelarg4dpu2kernel4dpu as target2function4dpu
#</function4xi>
 
#<function4xi function = "importtrace2kernel4dpu2kernel4dpu" usage = " importing target function for Process4Xi">
def importtrace2kernel4dpu2kernel4dpu():
   global target2function4dpu
   from dpu2kernel4xi import trace2kernel4dpu2kernel4dpu as target2function4dpu
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
# Test modules of DPU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importdpu2commandqueue4xi5test" usage = " importing target function for Process4Xi">
def importdpu2commandqueue4xi5test():
   global target2function4dpu
   from dpu2commandqueue4xi5test import dpu2commandqueue4xi5test as target2function4dpu
#</function4xi>
 
#<function4xi function = "importdpu2event4xi5test" usage = " importing target function for Process4Xi">
def importdpu2event4xi5test():
   global target2function4dpu
   from dpu2event4xi5test import dpu2event4xi5test as target2function4dpu
#</function4xi>
 
#<function4xi function = "importdpu2image4xi5test" usage = " importing target function for Process4Xi">
def importdpu2image4xi5test():
   global target2function4dpu
   from dpu2image4xi5test import dpu2image4xi5test as target2function4dpu
#</function4xi> 

#<function4xi function = "importdpu3image4xi5test" usage = " importing target function for Process4Xi">
def importdpu3image4xi5test():
   global target2function4dpu
   from dpu3image4xi5test import dpu3image4xi5test as target2function4dpu
#</function4xi>
 
#<function4xi function = "importdpu2sampler4xi5test" usage = " importing target function for Process4Xi">
def importdpu2sampler4xi5test():
   global target2function4dpu
   from dpu2sampler4xi5test import dpu2sampler4xi5test as target2function4dpu
#</function4xi> 

#<function4xi function = "importdpu2kernel4xi5test" usage = " importing target function for Process4Xi">
def importdpu2kernel4xi5test():
   global target2function4dpu
   from dpu2kernel4xi5test import dpu2kernel4xi5test as target2function4dpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Error handler of DPU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "handle2error4dpu" usage = " Error handling for importing target function for Process4Xi">
def handle2error4dpu():
   global import2use4dpu
   print ("HANDLE2ERROR4DPU OF DPU2PROGRAM4XI.PY: *** FAILED TO IMPORT TARGET FUNCTION ***")
   keys2import4dpu = import2use4dpu.keys()
   for key2import4dpu in keys2import4dpu:
       print(key2import4dpu,import2use4dpu[key2import4dpu]) 
#</function4xi> 


import2use4dpu = {}
# -------------------------------------------------------------------------------------------------
# Command queue module of DPU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4dpu["create2commandqueue4dpu"] = importcreate2commandqueue4dpu        
import2use4dpu["flush2commandqueue4dpu"] = importflush2commandqueue4dpu 
import2use4dpu["finish2commandqueue4dpu"] = importfinish2commandqueue4dpu 
import2use4dpu["get2commandqueueinfo4dpu"] = importget2commandqueueinfo4dpu 
import2use4dpu["release2commandqueue4dpu"] = importrelease2commandqueue4dpu 
import2use4dpu["retain2commandqueue4dpu"] = importretain2commandqueue4dpu 
import2use4dpu["set2commandqueue4dpu"] = importset2commandqueue4dpu 
import2use4dpu["trace2commandqueue4dpu"] = importtrace2commandqueue4dpu 

# -------------------------------------------------------------------------------------------------
# Context module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4dpu["create2context4dpu"] = importcreate2context4dpu 
import2use4dpu["get2context4dpu"] = importget2context4dpu 
import2use4dpu["release2context4dpu"] = importrelease2context4dpu 
import2use4dpu["retain2context4dpu"] = importretain2context4dpu 
import2use4dpu["set2context4dpu"] = importset2context4dpu 

# -------------------------------------------------------------------------------------------------
# Device module of DPU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4dpu["init2dpu4xi"] = importinit2dpu4xi 
import2use4dpu["set2dpu4xi"] = importset2dpu4xi 
import2use4dpu["get2dpu4xi"] = importget2dpu4xi 

# -------------------------------------------------------------------------------------------------
# Event module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4dpu["create2event4dpu"] = importcreate2event4dpu 
import2use4dpu["get2eventinfo4dpu"] = importget2eventinfo4dpu 
import2use4dpu["release2event4dpu"] = importrelease2event4dpu 
import2use4dpu["retain2event4dpu"] = importretain2event4dpu 
import2use4dpu["set2eventcallback4dpu"] = importset2eventcallback4dpu 
import2use4dpu["set2event4dpu"] = importset2event4dpu 
import2use4dpu["trace2event4dpu"] = importtrace2event4dpu 
import2use4dpu["wait2event4dpu"] = importwait2event4dpu 

# -------------------------------------------------------------------------------------------------
# 2D Image module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4dpu["create2image4dpu"] = importcreate2image4dpu 
import2use4dpu["get2imageinfo4dpu"] = importget2imageinfo4dpu 
import2use4dpu["get2supportedimageformats4dpu"] = importget2supportedimageformats4dpu 
import2use4dpu["manoeuvre2copyimage4dpu"] = importmanoeuvre2copyimage4dpu 
import2use4dpu["manoeuvre2copyimagetobuffer4dpu"] = importmanoeuvre2copyimagetobuffer4dpu 
import2use4dpu["manoeuvre2copybuffertoimage4dpu"] = importmanoeuvre2copybuffertoimage4dpu 
import2use4dpu["manoeuvre2readimage4dpu"] = importmanoeuvre2readimage4dpu 
import2use4dpu["manoeuvre2writeimage4dpu"] = importmanoeuvre2writeimage4dpu 
import2use4dpu["manoeuvre2mapimage4dpu"] = importmanoeuvre2mapimage4dpu 
import2use4dpu["copy2image4dpu"] = importcopy2image4dpu 
import2use4dpu["copy2imagetobuffer4dpu"] = importcopy2imagetobuffer4dpu 
import2use4dpu["copy2buffertoimage4dpu"] = importcopy2buffertoimage4dpu 
import2use4dpu["read2image4dpu"] = importread2image4dpu 
import2use4dpu["map2image4dpu"] = importmap2image4dpu 
import2use4dpu["release2image4dpu"] = importrelease2image4dpu 
import2use4dpu["retain2image4dpu"] = importretain2image4dpu 
import2use4dpu["set2imagecallback4dpu"] = importset2imagecallback4dpu 
import2use4dpu["set2image4dpu"] = importset2image4dpu 
import2use4dpu["trace2image4dpu"] = importtrace2image4dpu 
import2use4dpu["wait2image4dpu"] = importwait2image4dpu 
import2use4dpu["write2image4dpu"] = importwrite2image4dpu 

# -------------------------------------------------------------------------------------------------
# 3D Image module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4dpu["create3image4dpu"] = importcreate3image4dpu 
import2use4dpu["get3imageinfo4dpu"] = importget3imageinfo4dpu 
import2use4dpu["get3supportedimageformats4dpu"] = importget3supportedimageformats4dpu 
import2use4dpu["manoeuvre3copyimage4dpu"] = importmanoeuvre3copyimage4dpu 
import2use4dpu["manoeuvre3copyimagetobuffer4dpu"] = importmanoeuvre3copyimagetobuffer4dpu 
import2use4dpu["manoeuvre3copybuffertoimage4dpu"] = importmanoeuvre3copybuffertoimage4dpu 
import2use4dpu["manoeuvre3readimage4dpu"] = importmanoeuvre3readimage4dpu 
import2use4dpu["manoeuvre3writeimage4dpu"] = importmanoeuvre3writeimage4dpu 
import2use4dpu["manoeuvre3mapimage4dpu"] = importmanoeuvre3mapimage4dpu 
import2use4dpu["copy3image4dpu"] = importcopy3image4dpu 
import2use4dpu["copy3imagetobuffer4dpu"] = importcopy3imagetobuffer4dpu 
import2use4dpu["copy3buffertoimage4dpu"] = importcopy3buffertoimage4dpu 
import2use4dpu["read3image4dpu"] = importread3image4dpu 
import2use4dpu["map3image4dpu"] = importmap3image4dpu 
import2use4dpu["release3image4dpu"] = importrelease3image4dpu 
import2use4dpu["retain3image4dpu"] = importretain3image4dpu 
import2use4dpu["set3imagecallback4dpu"] = importset3imagecallback4dpu 
import2use4dpu["set3image4dpu"] = importset3image4dpu 
import2use4dpu["trace3image4dpu"] = importtrace3image4dpu 
import2use4dpu["wait3image4dpu"] = importwait3image4dpu 
import2use4dpu["write3image4dpu"] = importwrite3image4dpu 

# -------------------------------------------------------------------------------------------------
# Sampler module of DPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4dpu["create2sampler4dpu"] = importcreate2sampler4dpu 
import2use4dpu["get2samplerinfo4dpu"] = importget2samplerinfo4dpu 
import2use4dpu["release2sampler4dpu"] = importrelease2sampler4dpu 
import2use4dpu["retain2sampler4dpu"] = importretain2sampler4dpu 
import2use4dpu["set2samplercallback4dpu"] = importset2samplercallback4dpu 
import2use4dpu["set2sampler4dpu"] = importset2sampler4dpu 
import2use4dpu["trace2sampler4dpu"] = importtrace2sampler4dpu 

# -------------------------------------------------------------------------------------------------
# Test modules of DPU imports for target function                             
# -------------------------------------------------------------------------------------------------
import2use4dpu["dpu2commandqueue4xi5test"] = importdpu2commandqueue4xi5test 
import2use4dpu["dpu2event4xi5test"] = importdpu2event4xi5test 
import2use4dpu["dpu2image4xi5test"] = importdpu2image4xi5test 
import2use4dpu["dpu3image4xi5test"] = importdpu3image4xi5test 
import2use4dpu["dpu2sampler4xi5test"] = importdpu2sampler4xi5test 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2program4dpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF DPU2PROGRAM4XI5TEST - TRYING TO TRACE DPU: ', time4xi())      
#   print('=' *33)      
#   trace2dpu = None
#   try:
#       trace2dpu = trace2program4dpu() 
#       print('(Step 15B/25) OF DPU2PROGRAM4XI5TEST - TRACE2DPU: ', trace2dpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4dpu-failed {}; time: 04:32:25 11/07/11 Pacific Standard Time}', 'CLASS': 'trace2program4dpu', 'METHOD': 'trace2program4dpu'} 
#   except:
#       print('(Step 15Z1/25) OF DPU2PROGRAM4XI5TEST *** FAILED TO TRACE DPU: ', trace2dpu) 
#      
#">
# -------------------------------------------------------------------------------------------------   
def trace2program4dpu(task4trace2dpu=None):  

    '''
    creates callback to dpu2sourceprogram4xi
    '''
    
    global dpu2trace4xi


    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2program4dpu" 
    method2print = "trace2program4dpu"

    message2print = "{(Step 1A/33) started" + "; trace2program4dpu-started: " + str(trace2program4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function trace2program4dpu at 0x0000000003A202C8> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2dpu-to-trace2program4dpu: " + str(task4trace2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    dpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2dpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY DPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2dpu = {}                                                                                            
            
        task4trace2dpu['CLASS']   = "trace2program4dpu"                                                               # DPUIZE ZPU TRACE!
        task4trace2dpu['METHOD']  = "trace2program4dpu" 
        task4trace2dpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2program4dpu-failed " + str(dpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2dpu-set-by-trace2program4dpu to: " + str(task4trace2dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY DPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2dpu-try-by-trace2program4dpu: " + str(task4trace2dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 

    try:
        dpu2trace4xi ['CLASS']  = task4trace2dpu['CLASS']                                                      # DPUIZE DPU TRACE!
        dpu2trace4xi ['METHOD']  = task4trace2dpu['METHOD']
        dpu2trace4xi ['MESSAGE'] = task4trace2dpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; dpu2trace4xi-set-by-trace2program4dpu to: " + str(dpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; dpu-trace-failed-by-trace2program4dpu: " + str(task4trace2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE DPU TRACE
        # =================================
        class2print   = task4trace2dpu['CLASS']                                                                # DPUIZE DPU TRACE!
        method2print  = task4trace2dpu['METHOD']
        
        message2print = task4trace2dpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # DPUIZED TRACE FOR OPENCL CALLBACK!
        # 
        
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2program4dpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2program4dpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-dpu-trace-failed-by-trace2program4dpu: " + str(task4trace2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2program4dpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2program4dpu"                
        message2print = "{(Step 3D/33) performed" + "; dpu-trace-performed-by-trace2program4dpu: " + str(task4trace2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; dpu-trace-failed-by-trace2program4dpu: " + str(task4trace2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; dpu2trace4xi-returning-by-trace2program4dpu: " + str(dpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4dpu-failed {}; time: 04:40:23 11/16/11 Pacific Standard Time}', 
    #  'CLASS': 'trace2program4dpu', 
    #  'METHOD': 'trace2program4dpu'}; time: 04:40:23 11/16/11 Pacific Standard Time}

    
    return dpu2trace4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "unload2compiler4dpu" usage = "
#
#   response4unload2compiler = unload2compiler4dpu()
#      
#">
# -------------------------------------------------------------------------------------------------   
def unload2compiler4dpu(task4unload2dpu=None):  

    '''
    unloads compiler
    '''


            
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "unload2compiler4dpu" 
    method2print = "unload2compiler4dpu"

    message2print = "{(Step 1A/33) started" + "; unload2compiler4dpu-started: " + str(unload2compiler4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # 
  
          
    message2print = "{(Step 1B/33) passed" + "; task4unload2dpu-to-unload2compiler4dpu: " + str(task4unload2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                            # 
    # 
    status4unload2dpu = 'CL_SUCCESS'
    
    code2ack  = ' '
    code2nack = ' '    
    
                                                                                                                  #       
    if task4unload2dpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN - BUILD THEM
        # -----------------------------------------------------------------------------------------    
        task4unload2dpu = {}                                                                                            

        message2print = "{(Step 2A/33) set" + "; task4unload2dpu-set-by-unload2compiler4dpu to: " + str(task4unload2dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # 
    # -----------------------------------------------------------------------------------------          
    status4unload2dpu = 'CL_SUCCESS'
    message2print = "{(Step 3A/33) set" + "; status4unload2dpu-set-by-unload2compiler4dpu to: " + str(status4unload2dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; status4unload2dpu-returning-by-unload2compiler4dpu: " + str(status4unload2dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    
    return status4unload2dpu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programcallback4dpu" usage = "
#
#   print('(Step 14A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM CALLBACK OF DPU ', time4xi())  
#   try:                               
#       response4setcallback2program = set2programcallback4dpu()                                                                                                 
#       print('(Step 14B/700) OF DPU2PROGRAM4XI5TEST - SET PROGRAM CALLBACK OF DPU TO: ', response4setcallback2program)   
#       # 
#   except:
#       print('(Step 14Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM CALLBACK OF DPU ', time4xi())  
# 
#">
# ------------------------------------------------------------------------------------------------- 
 
def set2programcallback4dpu(task4set2programwithsourcecallback=None):

    '''
    registers callback function to dpu2program4xi
    '''
    global dpu2program4xi

            
    class2print = "set2programcallback4dpu" 
    method2print = "set2programcallback4dpu"
    
    message2print = "{(Step 1A/70) started" + "; set2programcallback4dpu-started: " + str(set2programcallback4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # set2programcallback4dpu-started
    
    try:
    
        dpu2program4xi ['CALLBACK'] = trace2program4dpu
        
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-dpu2program4xi-returning-by-set2programcallback4dpu: " + str(dpu2program4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2program4dpu at 0x0000000003A985C8>
        
        return dpu2program4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2program-failed-by-set2programcallback4dpu: " + str(callback2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2program4dpu-returning-by-set2programcallback4dpu: " + str(trace2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2program4dpu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "build2program4dpu" usage = "
#
#   task4build2program  = {}
#   task4build2program  ['PROGRAM'] = folder2lib4dpu + 'dpu2sampler4xi.py'
#   task4build2program  ['OPTIONS'] = {}
#   task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
#   task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
#   task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
#   task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'
#   print('(Step 99A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO BUILD PROGRAM OF DPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4build2program = build2program4dpu(task4build2program)
#       print('(Step 99B/700) OF DPU2PROGRAM4XI5TEST - BUILT PROGRAM OF DPU DEVICE ', time4xi())
#   except:
#       print('(Step 99Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO BUILD PROGRAM OF DPU DEVICE ', time4xi())  
#   try:                                                                                                                 
#       keys2response = response4build2program.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4build2program[key2response])
#           # ACK2RETURN  :   11A1-12A1-
#           # NACK2RETURN  :   13Z1-14Z1-
#           # OPTIONS  :  {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
#           # PROGRAM  :  folder2lib4dpu + 'dpu2sampler4xi.py'
#       
#   except:
#       print('(Step 99Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4BUILD2PROGRAM ', response4build2program) 
#                 
#">
# -------------------------------------------------------------------------------------------------   
def build2program4dpu(task4build2dpu=None):

    '''
    building program to DPU 
    '''
    global dpu2build4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "build2program4dpu"
    method2print = "build2program4dpu" 

    message2print = "{(Step 1A/33) started" + "; build2program4dpu-started: " + str(build2program4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # build2program4dpu-started
    
    
    message2print = "{(Step 1B/33) passed" + "; task4build2dpu-to-build2program4dpu: " + str(task4build2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'PROGRAM': '...//dpu2sampler4xi.py', 
    #  'OPTIONS': {'VERSION': 'CL1.1', 
    #  'OPTIMIZATION': 'DISABLE', 
    #  'MATH': 'SINGLE-PRECISION-CONSTANT', 
    #  'WARNINGS': 'INHIBIT'}}
    # ; time: 07:57:00 11/18/11 Pacific Standard Time}

             
    code2ack  = ' '
    code2nack = ' '

    message2print = "{(Step 1A/70) try" + "; build-try-by-build2program4dpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    try:             
        if task4build2dpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO DPU! 
            # -----------------------------------------------------------------------------------------    
            task4build2dpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4build2dpu-created-by-build2program4dpu: " + str(task4build2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1) SOURCE PROGRAM - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            program2dpu = {}
            message2print = "{(Step 1A/70) trying" + "; program2dpu-trying-by-create2programwithsource4dpu: " + str(program2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                program2dpu = create2programwithsource4dpu()                                                                     
                
                message2print = "{(Step 1B/70) got" + "; program2dpu-got-by-create2programwithsource4dpu: " + str(program2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '1B-'         
            except:
                message2print = "{(Step 1Z/70) failed" + "; program2dpu-failed-by-create2programwithsource4dpu: " + str(program2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '1Z-'
                
            task4build2dpu ['PROGRAM'] = program2dpu
            
            # ---------------------------------------------------------------------------------------------
            # (2) OPTIONS - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            task4build2program  ['OPTIONS'] = {}
            task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
            task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
            task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
            task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'            
            
            # ---------------------------------------------------------------------------------------------
            # (3) CALLBACK - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            function2callback4dpu = trace2program4dpu                                                               # CALLBACK FUNCTION FOR DPU
            
            message2print = "{(Step 3A/70) initialized" + "; function2callback4dpu-created-by-build2program4dpu: " + str(function2callback4dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '3A-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "build2program4dpu"                                                            # MODULE OF DPUIZED OPENCL MODULE
            args2callback['METHOD']  = "build2program4dpu"                                                            # FUNCTION OF DPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 3Z1/33) failed" + "; build2program4dpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DPUIZED OPENCL TO RETURN    

            message2print = "{(Step 3B/70) initialized" + "; args2callback-created-by-build2program4dpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '3B-'
            
           
            task4build2dpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO DPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4build2dpu['FUNCTION2CALLBACK'] = function2callback4dpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '3C-'

            
            # ---------------------------------------------------------------------------------------------
            # (4) USERDATA - REQUIRED                         
            # ---------------------------------------------------------------------------------------------
            task4build2dpu['USERDATA'] = None 
            
            message2print = "{(Step 4A/70) initialized" + "; task4build2dpu-created-by-build2program4dpu: " + str(task4build2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
        else:
            pass                                                                  
    except:
        message2print = "{(Step 1Z1/70) yielded" + "; task4build2dpu-yielded-by-build2program4dpu: " + str(task4build2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-'  
                                                                                                 
    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4build2dpu.keys()        
    message2print = "{(Step 9/70) using" + "; keys2task-using-by-build2program4dpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['OPTIONS', 'PROGRAM']
  
    # ---------------------------------------------------------------------------------------------
    # (Step 10A/70) - CREATE BUILD FOR DPU ! 
    # ---------------------------------------------------------------------------------------------    
    dpu2build4xi = {}                                                                                                                                                                               
    message2print = "{(Step 10A/70) initialized" + "; dpu2build4xi-initialized-by-build2program4dpu to: " + str(dpu2build4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}
    
    message2print = "{(Step 11A0/70) try" + "; program-try-by-build2program4dpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 11A/70) - PROGRAM - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        dpu2build4xi['PROGRAM'] = task4build2dpu['PROGRAM']
        
        message2print = "{(Step 11A1/70) initialized" + "; program-created-by-build2program4dpu: " + str(dpu2build4xi['PROGRAM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # ...//dpu2sampler4xi.py
        
        code2ack  = code2ack + '11A1-'
    except: 
        message2print = "{(Step 11Z1/70) yielded" + "; program-dpu2build4xi-yielded-by-build2program4dpu: " + str(dpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '11Z1-' 
           
    message2print = "{(Step 12A0/70) try" + "; options-try-by-build2program4dpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 12A/70) - OPTIONS - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        dpu2build4xi['OPTIONS'] = task4build2dpu['OPTIONS']
        
        message2print = "{(Step 12A1/70) initialized" + "; options-created-by-build2program4dpu: " + str(dpu2build4xi['OPTIONS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
        
        code2ack  = code2ack + '12A1-'
    except: 
        message2print = "{(Step 12Z1/70) yielded" + "; options-dpu2build4xi-yielded-by-build2program4dpu: " + str(dpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '12Z1-' 
        
    message2print = "{(Step 13A0/70) try" + "; callback-try-by-build2program4dpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 13A/70) - CALLBACK - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        dpu2build4xi['CALLBACK'] = task4build2dpu['CALLBACK']
        
        message2print = "{(Step 13A1/70) initialized" + "; callback-created-by-build2program4dpu: " + str(dpu2build4xi['CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '13A1-'
    except: 
        message2print = "{(Step 13Z1/70) yielded" + "; callback-dpu2build4xi-yielded-by-build2program4dpu: " + str(dpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '13Z1-' 
        
    message2print = "{(Step 14A0/70) try" + "; userdata-try-by-build2program4dpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 14A/70) - USERDATA - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        dpu2build4xi['USERDATA'] = task4build2dpu['USERDATA']
        
        message2print = "{(Step 14A1/70) initialized" + "; userdata-created-by-build2program4dpu: " + str(dpu2build4xi['USERDATA'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '14A1-'
    except: 
        message2print = "{(Step 14Z1/70) yielded" + "; userdata-dpu2build4xi-yielded-by-build2program4dpu: " + str(dpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '14Z1-'
                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2build4xi ['ACK2RETURN']  = code2ack
    dpu2build4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; dpu2build4xi-returning-by-build2program4dpu: " + str(dpu2build4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'ACK2RETURN': ' 11A1-12A1-', 
    #  'PROGRAM': '...//dpu2sampler4xi.py', 
    #  'NACK2RETURN': ' 13Z1-14Z1-', 
    #  'OPTIONS': {'VERSION': 'CL1.1', 
    #  'OPTIMIZATION': 'DISABLE', 
    #  'MATH': 'SINGLE-PRECISION-CONSTANT', 
    #  'WARNINGS': 'INHIBIT'}
    # }; time: 07:57:00 11/18/11 Pacific Standard Time}

    
    return dpu2build4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2programwithsource4dpu" usage = "
#     
#   print('(Step 2A1/700) OF DPU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM SOURCE FOR DPU ', time4xi())       
#   try:
#       response4create2programwithsource = create2programwithsource4dpu()
#   except:
#       print('(Step 2Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM SOURCE FOR DPU *** ', time4xi())
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4create2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4create2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4create2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4create2programwithsource['SIZE4SOURCE'])
#       keys2source = response4create2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [3])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [4])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [5])
#           
#       # COUNT4SOURCE:  1
#       # DIRECTORY4SOURCE:  folder2lib4dpu
#       # LANGUAGE4SOURCE:  Python
#       # SIZE4SOURCE:  {folder2lib4dpu + 'dpu2sampler4xi5test.py': 1337}
#       # SOURCE:  folder2lib4dpu + 'dpu2sampler4xi5test.py'
#       # ... profile = "11" />
#       # ... program = "dpu2sampler4xi5test.py">
#       # ... abstract = "tests dpu2sampler4xi.py"/>
#       # ... verification = "11" target = "dpu2sampler4xi.py"/>                
#   except:
#       print('(Step 2Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithsource *** ', response4create2programwithsource)
#          
#">
# -------------------------------------------------------------------------------------------------   
def create2programwithsource4dpu(task4program2dpu=None):

    '''
    creates program for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    

    global dir2source4dpu                                                                                           # Directory for source files      


    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2programwithsource4dpu" 
    method2print = "create2programwithsource4dpu"

    message2print = "{(Step 1A/70) started" + "; create2programwithsource4dpu-started: " + str(create2programwithsource4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2programwithsource4dpu at 0x00000000040D77C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2dpu-to-create2programwithsource4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    dpu2program4xi = {}                                                                                              # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4program2dpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO DPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4program2dpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4program2dpu-created-by-create2programwithsource4dpu: " + str(task4program2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4dpu = trace2program4dpu                                                               # CALLBACK FUNCTION FOR DPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4dpu-created-by-create2programwithsource4dpu: " + str(function2callback4dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2program4dpu at 0x0000000003AB12C8>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2programwithsource4dpu"                                               # MODULE OF DPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2programwithsource4dpu"                                               # FUNCTION OF DPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2programwithsource4dpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2programwithsource4dpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4dpu-failed ; time: 06:57:23 11/15/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4dpu', 
            #  'METHOD': 'create2programwithsource4dpu'}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4program2dpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO DPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4program2dpu['FUNCTION2CALLBACK'] = function2callback4dpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4program2dpu-created-by-create2programwithsource4dpu: " + str(task4program2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2program4dpu at 0x0000000003AB12C8>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4dpu-failed ; time: 06:57:23 11/15/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4dpu', 
            #  'METHOD': 'create2programwithsource4dpu'}}; time: 06:57:23 11/15/11 Pacific Standard Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2programwithsource4dpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2programwithsource4dpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 06:57:23 11/15/11 Pacific Standard Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2programwithsource4dpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4program2dpu['PLATFORM'] = platform2xi                                                              # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2dpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2dpu-trying-by-create2programwithsource4dpu: " + str(context2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dpu = get2context4dpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dpu-got-by-create2programwithsource4dpu: " + str(context2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dpu-failed-by-create2programwithsource4dpu: " + str(context2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4program2dpu ['CONTEXT'] = context2dpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2dpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dpu-trying-by-create2programwithsource4dpu: " + str(device2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dpu = get2dpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dpu-got-by-create2programwithsource4dpu: " + str(device2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dpu-failed-by-create2programwithsource4dpu: " + str(device2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4program2dpu ['DEVICE'] = device2dpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4program2dpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2dpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2dpu-trying-by-create2programwithsource4dpu: " + str(queue2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2dpu-got-by-create2programwithsource4dpu: " + str(queue2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003ACCF98>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2dpu-failed-by-create2programwithsource4dpu: " + str(queue2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4program2dpu ['QUEUE'] = queue2dpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2dpu-trying-by-create2programwithsource4dpu: " + str(pipe2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2dpu-got-by-create2programwithsource4dpu: " + str(pipe2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2dpu-failed-by-create2programwithsource4dpu: " + str(pipe2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4program2dpu ['PIPE'] = pipe2dpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2dpu-trying-by-create2programwithsource4dpu: " + str(commandqueueinfo2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dpu = get2commandqueueinfo4dpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2dpu-got-by-create2programwithsource4dpu: " + str(commandqueueinfo2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2dpu-failed-by-create2programwithsource4dpu: " + str(commandqueueinfo2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4program2dpu ['COMMANDQUEUE'] = commandqueueinfo2dpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2dpu-trying-by-create2programwithsource4dpu: " + str(bufferinfo2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dpu = get2bufferinfo4dpu()                                                               # SYSTEM CREATES BUFFER FOR DPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2dpu-got-by-create2programwithsource4dpu: " + str(bufferinfo2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2dpu-failed-by-create2programwithsource4dpu: " + str(bufferinfo2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4program2dpu ['BUFFER'] = bufferinfo2dpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2dpu-trying-by-create2programwithsource4dpu: " + str(eventinfo2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dpu = get2eventinfo4dpu()                                                                 # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2dpu-got-by-create2programwithsource4dpu: " + str(eventinfo2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2dpu-failed-by-create2programwithsource4dpu: " + str(eventinfo2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4program2dpu ['EVENT'] = eventinfo2dpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) COUNT - REQUIRED                     
            # ---------------------------------------------------------------------------------------------             
            task4program2dpu ['COUNT'] = 0
            message2print = "{(Step 11A/70) trying" + "; count2dpu-trying-by-create2programwithsource4dpu: " + str(task4program2dpu['COUNT'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:            
                task4program2dpu ['COUNT'] = int(len(kb2fetch4dpu)/2)                                               # NUMBER OF SOURCE FILES TO BE READ AND EMBEDDED
                message2print = "{(Step 11B/70) set" + "; count2dpu-set-by-create2programwithsource4dpu to: " + str(task4program2dpu['COUNT'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 1                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; count2dpu-failed-by-create2programwithsource4dpu: " + str(task4program2dpu['COUNT']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
                                               
            # ---------------------------------------------------------------------------------------------
            # (12) DIRECTORY - REQUIRED - COMMON TO ALL KIND OF SOURCE FILES (Python, C++, C ...)                 
            # ---------------------------------------------------------------------------------------------             
            dir2dpu = dir2source4dpu                                                                                                                                                          
            message2print = "{(Step 12B/70) set" + "; dir2dpu-set-by-create2programwithsource4dpu to: " + str(dir2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  
            code2ack  = code2ack + '12B-' 
        
            task4program2dpu ['DIRECTORY'] = dir2dpu
            
                        
            # ---------------------------------------------------------------------------------------------
            # (14) CODE AND SIZE - REQUIRED                 
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A/70) using" + "; DirectoryWalker-using-by-create2programwithsource4dpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # <class 'dpu2program4xi.DirectoryWalker'>
            
            for file2walk in DirectoryWalker(dir2dpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B/70) examining" + "; directory-examining-by-create2programwithsource4dpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C/70) examining" + "; file-examining-by-create2programwithsource4dpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4dpu +  dpu2buffer4xi.py
                    
                    message2print = "{(Step 14D/70) calling" + "; fetch2programsource4dpu-calling-by-create2programwithsource4dpu: " + str(fetch2programsource4dpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2programsource4dpu at 0x0000000003FE18C8>
                    
                    message2print = "{(Step 14E/70) passing" + "; file2walk-passing-to-fetch2programsource4dpu-by-create2programwithsource4dpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #folder2lib4dpu +  dpu2buffer4xi.py
                    
                    fetch2programsource4dpu(file2walk)                                                              # fetches program files into fetch2code4source dictionary 
                    
                    message2print = "{(Step 14E/70) performed" + "; fetch2programsource4dpu-performed-by-create2programwithsource4dpu: " + str(fetch2programsource4dpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 
                    
                    message2print = "{(Step 14F/70) got" + "; fetch2code4source-got-by-create2programwithsource4dpu: " + str(fetch2code4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 
                                        
            task4program2dpu ['CODE'] = fetch2code4source                                                           # dictionary of source code(s)      
                    
            size2source4dpu  = {}
            keys2source4dpu = fetch2code4source.keys()                                                              # dpu2sampler4xi5test.py
            for key2source4dpu in keys2source4dpu:
                size2source4dpu[key2source4dpu] = len(fetch2code4source[key2source4dpu])                            # length of saved source   
                       
            message2print = "{(Step 14B/70) set" + "; size2source4dpu-set-by-create2programwithsource4dpu to: " + str(size2source4dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # {folder2lib4dpu + 'dpu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '14B-' 
        
            task4program2dpu ['SIZE'] = size2source4dpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            language2dpu  = "Python"  # C++; C
           
            message2print = "{(Step 15B/70) set" + "; language2dpu-set-by-create2programwithsource4dpu to: " + str(language2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '15B-' 
        
            task4program2dpu ['LANGUAGE'] = language2dpu     

                                    
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2dpu  = 1
           
            message2print = "{(Step 17B/70) set" + "; count2reference2dpu-set-by-create2programwithbinary4dpu to: " + str(count2reference2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '17B-' 
        
            task4program2dpu ['COUNT2REFERENCE'] = count2reference2dpu                                 
                                                                                                                      
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2dpu-yielded-by-create2programwithsource4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4program2dpu.keys()         
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2programwithsource4dpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'COUNT',
    #  'COUNT2REFERENCE',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'DIRECTORY',
    #  'EVENT',
    #  'FUNCTION2CALLBACK',
    #  'LANGUAGE',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'SIZE']
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR DPU ! 
    # ---------------------------------------------------------------------------------------------    
    dpu2program4xi = {}                                                                                             # CREATE INITIALIZES DICTIONARY!                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; dpu2program4xi-initialized-by-create2programwithsource4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 

    if 'DATA2CALLBACK' in keys2task:    
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - EXTRA 
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['DATA2CALLBACK'] = task4program2dpu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4dpu-failed ; time: 01:21:23 11/16/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4dpu', 
            #  'METHOD': 'create2programwithsource4dpu'}; time: 01:21:23 11/16/11 Pacific Standard Time}
     
            
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'

    if 'FUNCTION2CALLBACK' in keys2task:     
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['FUNCTION2CALLBACK'] = task4program2dpu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2program4dpu at 0x000000000392E2C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 

    if 'PLATFORM' in keys2task:     
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['PLATFORM'] = task4program2dpu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 01:21:23 11/16/11 Pacific Standard Time}
    
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
        

    if 'CONTEXT' in keys2task:         
          
        message2print = "{(Step 23A/70) try" + "; context2dpu-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT - REQUIRED
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['CONTEXT'] = task4program2dpu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...        
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 

    if 'DEVICE' in keys2task:         
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['DEVICE'] = task4program2dpu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ... 
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'

    if 'CODE2ERROR' in keys2task:                  
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['CODE2ERROR'] = task4program2dpu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0 
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'                

    if 'QUEUE' in keys2task:                 
              
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['QUEUE'] = task4program2dpu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003EEC978> 
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z1/70) yielded" + "; queue-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-' 

    if 'PIPE' in keys2task:                                 
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['PIPE'] = task4program2dpu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>) 
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-dpu2program4xi-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'   

    if 'COMMANDQUEUE' in keys2task: 
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['COMMANDQUEUE']  = task4program2dpu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, ... 
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    

    if 'BUFFER' in keys2task: 
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['BUFFER']  = task4program2dpu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            

    if 'EVENT' in keys2task: 
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['EVENT']  = task4program2dpu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK':  ... 
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'            

    if 'COUNT' in keys2task:                 
        message2print = "{(Step 31A1/70) try" + "; task4program2dpu-try-by-create2programwithsource4dpu: " + str(task4program2dpu)  + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # {'COUNT': 1, 'CODE2ERROR': 0 ... 
        message2print = "{(Step 31A2/70) try" + "; count-try-by-create2programwithsource4dpu: " + str(task4program2dpu['COUNT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1 
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['COUNT4SOURCE']  = task4program2dpu['COUNT']  
    
            message2print = "{(Step 31B/70) initialized" + "; count-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31B-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     

    if 'DIRECTORY' in keys2task: 
        message2print = "{(Step 32A/70) try" + "; directory-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['DIRECTORY4SOURCE']  = task4program2dpu['DIRECTORY']  
     
            message2print = "{(Step 32B/70) initialized" + "; directory-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #
            code2ack  = code2ack + '32B-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; DIRECTORY-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'  

    if 'CODE' in keys2task:         
        message2print = "{(Step 33A/70) try" + "; codetask-try-by-create2programwithsource4dpu: " + str(task4program2dpu ['CODE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ...
        
        dpu2program4xi['CODE4SOURCE'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['CODE4SOURCE'] = task4program2dpu ['CODE']
    
     
            message2print = "{(Step 33B/70) created" + "; code-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2:
             
            code2ack  = code2ack + '33B-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; source-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '33Z1-'     

    if 'SIZE' in keys2task:         
        message2print = "{(Step 34A/70) try" + "; size-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['SIZE4SOURCE']  = task4program2dpu['SIZE']  
     
            message2print = "{(Step 34B/70) initialized" + "; size-initialized-by-create2programwithsource4dpu to: " + str(dpu2program4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4dpu + 'dpu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '34B-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     

    if 'LANGUAGE' in keys2task:         
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['LANGUAGE4SOURCE']  = task4program2dpu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'     

    if 'COUNT2REFERENCE' in keys2task:                 
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['COUNT2REFERENCE']  = task4program2dpu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2programwithsource4dpu: " + str(dpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-'     
        
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)     
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-create2programwithsource4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE
        # SIZE4SOURCE
        # DATA2CALLBACK
        # BUFFER
        # FUNCTION2CALLBACK
        # ACK2RETURN
        # DIRECTORY4SOURCE
        # QUEUE
        # PLATFORM
        # COUNT2REFERENCE
        # NACK2RETURN
        # CONTEXT
        # COMMANDQUEUE
        # DEVICE
        # CODE4SOURCE
        # EVENT
        # PIPE

    return dpu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2programwithbinary4dpu" usage = "
#     
#   print('(Step 3A2/700) OF DPU2PROGRAM4XI5TEST - TRYING TO ADD BINARY INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4add2program = {}
#       task4add2program ['CODE4BINARY'] = {}
#       # NOT OK - EXISTS
#       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] = {}
#       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
#       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
#       task4add2program ['SIZE4BINARY'] = {}
#       task4add2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'])
#                
#       # OK
#       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = {}
#       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
#       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
#       task4add2program ['SIZE4BINARY'] = {}
#       task4add2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'])
#       
#       response4add2programwithbinary = add2programwithbinary4dpu(task4add2program)
#       print('(Step 3A3/700) OF DPU2PROGRAM4XI5TEST - ADDED BINARY INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithbinary: ', response4add2programwithbinary)
#   try:                                                                                          
#       keys2response = response4add2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4add2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4add2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4add2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4add2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4add2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4add2programwithbinary ['SIZE4BINARY'])                
#       # 
#       keys2source = response4add2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4add2programwithbinary ['CODE4BINARY'] [key2source] [1])
#           # 
#           # COUNT2REFERENCE: 1
#           # COUNT4BINARY: 2
#           # DIRECTORY4BINARY: ...//__pycache__//
#           # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000003F96AC8>
#           # LANGUAGE4BINARY: Python
#           # SIZE4BINARY: {'...//__pycache__//dpu2image4xi.cpython-33.pyc': 1006, '...//__pycache__//dpu3image4xi.cpython-33.pyc': 2}
#           # BINARY:  ...//__pycache__//dpu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'
#           # BINARY:  ...//__pycache__//dpu3image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'                       
#   except:
#       print('(Step 3Z3/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithbinary : ', response4add2programwithbinary)
#          
#">
# -------------------------------------------------------------------------------------------------   
def add2programwithbinary4dpu(task4program2dpu):

    '''
    adds program for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    
    
  
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2programwithbinary4dpu" 
    method2print = "add2programwithbinary4dpu"

    message2print = "{(Step 10A/70) started" + "; add2programwithbinary4dpu-started: " + str(add2programwithbinary4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function add2programwithbinary4dpu at 0x000000000418DC48>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2dpu-to-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4BINARY': {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
    #                  folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
    #  'SIZE4BINARY': {folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': 2}
    # }
    
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2binary4task = task4program2dpu ['CODE4BINARY'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2binary4task-using-by-add2programwithbinary4dpu: " + str(keys2binary4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # //__pycache__//dpu2image4xi.cpython-33.pyc
        # //__pycache__//dpu3image4xi.cpython-33.pyc          
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2binary4task-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  dpu2program4xi ['CODE4BINARY'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2programwithbinary4dpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4dpu = create2programwithbinary4dpu()
            keys2createwb4dpu = response2createwb4dpu.keys()
            for key2createwb4dpu in keys2createwb4dpu:
                if key2createwb4dpu == 'CODE4BINARY':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4dpu-created-by-add2programwithbinary4dpu: " + str(key2createwb4dpu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # {'... //__pycache__//dpu2image4xi.cpython-33.pyc'
            
            keys2exist4task =  dpu2program4xi ['CODE4BINARY'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2programwithbinary4dpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # //__pycache__//dpu2image4xi.cpython-33.pyc'
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; dpu2program4xi-failed-by-add2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL BINARY CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2binary4task in keys2binary4task:
            try:
                message2print = "{(Step 33A/70) using" + "; key2binary4task-using-by-add2programwithbinary4dpu: " + str(key2binary4task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # //__pycache__//dpu2image4xi.cpython-33.pyc
                            
                if key2binary4task in keys2exist4task:
                    message2print = "{(Step 33Z0/70) yielded" + "; key2binary4task-yielded-by-add2programwithbinary4dpu to: " + str(key2binary4task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4dpu +  __pycache__//dpu2image4xi.cpython-33.pyc
                     
                    code2nack  = code2nack + '33Z0-'
                else: 
                    try:
                        dpu2program4xi ['CODE4BINARY'] [key2binary4task] = task4program2dpu ['CODE4BINARY'] [key2binary4task]
                    except:
                        message2print = "{(Step 33Z1/70) failed" + "; task4program2dpu-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z1-' 
                    try:             
                        message2print = "{(Step 33B/70) added" + "; key2binary4task-added-by-add2programwithbinary4dpu to: " + str(key2binary4task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #folder2lib4dpu +  __pycache__//dpu3image4xi.cpython-33.pyc
                    except:
                        message2print = "{(Step 33Z2/70) failed" + "; task4program2dpu-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z2-' 
                    try:             
                        message2print = "{(Step 33C/70) added" + "; code-added-by-add2programwithbinary4dpu to: " + str(dpu2program4xi ['CODE4BINARY'] [key2binary4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}
                    except:
                        message2print = "{(Step 33Z3/70) failed" + "; task4program2dpu-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        code2nack  = code2nack + '33Z3-'                 
                    try:
                        dpu2program4xi ['COUNT4BINARY'] = dpu2program4xi ['COUNT4BINARY'] + 1
                        message2print = "{(Step 33D/70) added" + "; count4binary-added-by-add2programwithbinary4dpu to: " + str(dpu2program4xi ['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 2 
                    except:
                        message2print = "{(Step 33Z4/70) failed" + "; task4program2dpu-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z4-' 
                    try:                            
                        dpu2program4xi ['SIZE4BINARY'] [key2binary4task]  = task4program2dpu['SIZE4BINARY'] [key2binary4task]  
                        message2print = "{(Step 33E/70) set" + "; size4binary-set-by-add2programwithbinary4dpu to: " + str(dpu2program4xi ['SIZE4BINARY'] [key2binary4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 2
                        code2ack  = code2ack + '33D-'
                    except:
                        message2print = "{(Step 33Z5/70) failed" + "; task4program2dpu-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z5-'    
            except:
                message2print = "{(Step 33Z6/70) failed" + "; task4program2dpu-failed-by-add2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                code2nack  = code2nack + '33Z6-'                     
    except: 
        message2print = "{(Step 33Z7/70) yielded" + "; binary-yielded-by-add2programwithbinary4dpu: " + str(dpu2program4xi ['CODE4BINARY']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '33Z7-'       
          
                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2binaryprograms = dpu2program4xi ['CODE4BINARY'].keys()
    for key2binaryprogram in sorted(keys2binaryprograms):
        message2print = "{(Step 70A/70) got" + "; key2binaryprogram-got-by-add2programwithbinary4dpu: " + str(key2binaryprogram) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4dpu +  __pycache__//dpu2image4xi.cpython-33.pyc
        #folder2lib4dpu +  __pycache__//dpu3image4xi.cpython-33.pyc

    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4dpu-returning-by-add2programwithbinary4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
        
    return dpu2program4xi
            
#</function4xi>



# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2programwithsource4dpu" usage = "
#     
#   print('(Step 2A3/700) OF DPU2PROGRAM4XI5TEST - TRYING TO ADD SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())          
#   try:
#       task4add2program = {}
#       # OK
#       task4add2program ['CODE4SOURCE'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] [1] = '... profile = "33" />\n'
#       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] [2] = '... program = "dpu2buffer4xi.py" />\n'
#       task4add2program ['SIZE4SOURCE'] = {}
#       task4add2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'])
#       # NOT OK - EXISTS
#       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] [1] = '... profile = "33" />\n'
#       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] [2] = '... program = "dpu2sampler4xi5test.py" />\n'
#       
#       task4add2program ['SIZE4SOURCE'] = {}
#       task4add2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'])            
#       task4add2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'])            
#       
#       response4add2programwithsource = add2programwithsource4dpu(task4add2program)
#       print('(Step 2A4/700) OF DPU2PROGRAM4XI5TEST - ADDED SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#       #                                                                                   
#   except:
#       print('(Step 2Z3/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO ADD SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4add2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4add2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4add2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4add2programwithsource['SIZE4SOURCE'])
#       keys2source = response4add2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4add2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           # COUNT4SOURCE:  2
#           # DIRECTORY4SOURCE:  folder2lib4dpu
#           # LANGUAGE4SOURCE:  Python
#           # SIZE4SOURCE:  {'folder2lib4dpu + dpu2buffer4xi.py': 2, 'folder2lib4dpu + dpu2sampler4xi5test.py': 1337}
#           # SOURCE:  folder2lib4dpu + dpu2buffer4xi.py
#           # ... profile = "33" />
#           # 
#           # SOURCE:  folder2lib4dpu + dpu2sampler4xi5test.py
#           # ... profile = "11" />           
#   except:
#       print('(Step 2Z4/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithsource *** ', response4add2programwithsource)     
#          
#">
# -------------------------------------------------------------------------------------------------   
def add2programwithsource4dpu(task4program2dpu): 

    '''
    adds program for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    

    global dir2source4dpu                                                                                           # Directory for source files      

    
        
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2programwithsource4dpu" 
    method2print = "add2programwithsource4dpu"

    message2print = "{(Step 10A/70) started" + "; add2programwithsource4dpu-started: " + str(add2programwithsource4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function add2programwithsource4dpu at 0x000000000418DC48>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2dpu-to-add2programwithsource4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "dpu2buffer4xi.py" />\n'}, 
    #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "dpu2sampler4xi5test.py" />\n'}}, 
    #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 2, 
    #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': 2}}
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2source4task = task4program2dpu ['CODE4SOURCE'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2source4task-using-by-add2programwithsource4dpu: " + str(keys2source4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
              
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2source4task-failed-by-add2programwithsource4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  dpu2program4xi ['CODE4SOURCE'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2programwithsource4dpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4dpu = create2programwithsource4dpu()
            keys2createwb4dpu = response2createwb4dpu.keys()
            for key2createwb4dpu in keys2createwb4dpu:
                if key2createwb4dpu == 'CODE4SOURCE':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4dpu-created-by-add2programwithsource4dpu: " + str(key2createwb4dpu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # CODE4SOURCE 
            
            keys2exist4task =  dpu2program4xi ['CODE4SOURCE'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2programwithsource4dpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # dict_keys(['...//dpu2sampler4xi5test.py'])
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; dpu2program4xi-failed-by-add2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL SOURCE CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2source4task in keys2source4task:
            if key2source4task in keys2exist4task:
                message2print = "{(Step 33Z0/70) yielded" + "; key2source4task-yielded-by-add2programwithsource4dpu to: " + str(key2source4task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #folder2lib4dpu +  dpu2sampler4xi5test.py
                 
                code2nack  = code2nack + '33Z0-'
            else: 
                dpu2program4xi ['CODE4SOURCE'] [key2source4task] = task4program2dpu ['CODE4SOURCE'] [key2source4task]
 
                message2print = "{(Step 33B/70) added" + "; key2source4task-added-by-add2programwithsource4dpu to: " + str(key2source4task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #folder2lib4dpu +  dpu2buffer4xi.py
 
                message2print = "{(Step 33C/70) added" + "; code-added-by-add2programwithsource4dpu to: " + str(dpu2program4xi ['CODE4SOURCE'] [key2source4task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {1: '... profile = "33" />\n', 2: '... program = "dpu2buffer4xi.py" />\n'}
         
            
                dpu2program4xi ['COUNT4SOURCE'] = dpu2program4xi ['COUNT4SOURCE'] + 1
                message2print = "{(Step 33D/70) added" + "; count4source-added-by-add2programwithsource4dpu to: " + str(dpu2program4xi ['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 2
                
                dpu2program4xi ['SIZE4SOURCE'] [key2source4task]  = task4program2dpu['SIZE4SOURCE'] [key2source4task]  
                message2print = "{(Step 33E/70) set" + "; size4source-set-by-add2programwithsource4dpu to: " + str(dpu2program4xi ['SIZE4SOURCE'] [key2source4task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 2
                
                         
                code2ack  = code2ack + '33D-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; source-yielded-by-add2programwithsource4dpu: " + str(dpu2program4xi ['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # dpu2sampler4xi5test.py
        
        code2nack  = code2nack + '33Z1-'     
        

                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2sourceprograms = dpu2program4xi ['CODE4SOURCE'].keys()
    for key2sourceprograms in sorted(keys2sourceprograms):
        message2print = "{(Step 70A/70) got" + "; key2sourceprograms-got-by-add2programwithsource4dpu: " + str(key2sourceprograms) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4dpu +  dpu2buffer4xi.py
        #folder2lib4dpu +  dpu2sampler4xi5test.py
          
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4dpu-returning-by-add2programwithsource4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE
        # SIZE4SOURCE
        # DATA2CALLBACK
        # BUFFER
        # FUNCTION2CALLBACK
        # ACK2RETURN
        # DIRECTORY4SOURCE
        # QUEUE
        # PLATFORM
        # COUNT2REFERENCE
        # NACK2RETURN
        # CONTEXT
        # COMMANDQUEUE
        # DEVICE
        # CODE4SOURCE
        # EVENT
        # PIPE 

    return dpu2program4xi
            
#</function4xi>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "compile2program4dpu" usage = "
#     
#   print('(Step 2A2/700) OF DPU2PROGRAM4XI5TEST - TRYING TO COMPILE SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4compile = compile2program4dpu()
#   except:
#       print('(Step 2Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO COMPILE SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                      
#   try:
#       print('(Step 2B2/700) OF DPU2PROGRAM4XI5TEST - RESPONSE4COMPILE: ', time4xi())                                                                                      
#       keys2response4compile = response4compile.keys()
#       for key2response4compile in keys2response4compile:
#           print('KEY2RESPONSE4COMPILE: ', key2response4compile)
#           # dpu2sampler4xi5test.py            
#   except:
#       print('(Step 2Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4compile *** ', response4compile)
#   tailor2trace4program(t2xi)                                                                              # TAILOR TRACE AGAIN - TARGET FUNCTIONS OF ACTIVATED PROCESSES MIGHT HAVE RESET TRACE ! 
#          
#">
# -------------------------------------------------------------------------------------------------   
def compile2program4dpu(task4program2dpu=None):

    '''
    compiles program for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    
    global import2use4dpu                                                                                           # Imports of DPU
    global target2function4dpu                                                                                      # Target function of Process4Xi
    

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "compile2program4dpu" 
    method2print = "compile2program4dpu"

    message2print = "{(Step 1A/70) started" + "; compile2program4dpu-started: " + str(compile2program4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function compile2program4dpu at 0x00000000039D5E48>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2dpu-to-compile2program4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    source2program4dpu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1C/70) initialized" + "; source2program4dpu-initialized-by-compile2program4dpu to: " + str(source2program4dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #                
    try:             
        if task4program2dpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2dpu-yielded-by-compile2program4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - COMPILE SOURCES INSIDE PROGRAM OBJECT FOR DPU ! 
    # ---------------------------------------------------------------------------------------------    
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                            
        # ---------------------------------------------------------------------------------------------             
        keys2source4dpu = dpu2program4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2source4dpu-using-by-compile2program4dpu: " + str(keys2source4dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4dpu + 'dpu2sampler4xi5test.py'])
        
        unit4processing = "dpu"
        
        for key2source4dpu in keys2source4dpu:
            index2search=str.find(key2source4dpu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2compile = key2source4dpu[index2search:]                                                          # dpu2sampler4xi5test.py
                index2search=str.find(target2compile,".py")
                if index2search == -1:
                    pass
                else:
                    target2compile4dpu = target2compile[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2compile4dpu-set-by-compile2program4dpu to: " + str(target2compile4dpu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # dpu2sampler4xi5test

                    p2source4dpu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4DPU 
                        # ---------------------------------------------------------------------------------------------    
             
                        try:
                            import2use4dpu.get(target2compile4dpu, handle2error4dpu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2compile4dpu-imported-by-compile2program4dpu: " + str(target2compile4dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # dpu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4dpu-failed-by-compile2program4dpu: " + str(import2use4dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2compile4dpu-used-by-compile2program4dpu: " + str(target2compile4dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4DPU
                        # ---------------------------------------------------------------------------------------------                          
                        p2dpu = Process4Xi(target=target2function4dpu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2dpu-created-by-compile2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-1, initial)>
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2dpu.start()
                            message2print = "{(Step 34A/70) started" + "; p2dpu-started-by-compile2program4dpu: " + str(p2dpu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-1
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4dpu = p2dpu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-compile2program4dpu: " + str(status2process4dpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2dpu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2dpu-run-by-compile2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-compile2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-compile2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-compile2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2source4dpu-failed-by-compile2program4dpu: " + str(p2source4dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    
            # ---------------------------------------------------------------------------------------------
            # SAVE SOURCE WITHOUT END OF LINES
            # ---------------------------------------------------------------------------------------------        
            keys2lines = dpu2program4xi['CODE4SOURCE'][key2source4dpu].keys()     
            source2program4dpu[key2source4dpu] = ""
            for key2line in keys2lines:
                code2add4dpu = dpu2program4xi ['CODE4SOURCE'] [key2source4dpu] [key2line]
                source2program4dpu[key2source4dpu] = source2program4dpu[key2source4dpu] + code2add4dpu.replace("\n","")
        
            message2print = "{(Step 34A/70) saved" + "; lenght-source4dpu-saved-by-compile2program4dpu to: " + str(len(source2program4dpu[key2source4dpu]))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #     
                
 
        message2print = "{(Step 38A/70) set" + "; lenght-source2program4dpu-set-by-compile2program4dpu to: " + str(len(source2program4dpu))  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; source-yielded-by-compile2program4dpu: " + str(dpu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2source4dpu = source2program4dpu.keys()
    for key2source4dpu in keys2source4dpu:
        message2print = "{(Step 70/70) returning" + "; key2source4dpu-returning-by-compile2program4dpu: " + str(key2source4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        # 
    
    return source2program4dpu
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2programwithbinary4dpu" usage = "
#     
#   print('(Step 3A3/700) OF DPU2PROGRAM4XI5TEST - TRYING TO DELETE BINARY INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2program = {}
#       task4delete2program ['CODE4BINARY'] = {}
#       task4delete2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = 'Yes'
#       task4delete2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
#            task4delete2program ['SIZE4BINARY'] = {}
#       task4delete2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = 'Yes'
#       task4delete2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
#       
#       response4delete2programwithbinary = delete2programwithbinary4dpu(task4delete2program) 
#       print('(Step 3A3/700) OF DPU2PROGRAM4XI5TEST - DELETED BINARY INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO DELETE BINARY INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#   try:                                                                                          
#       keys2response = response4delete2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4delete2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4delete2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4delete2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4delete2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4delete2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4delete2programwithbinary ['SIZE4BINARY'])                
#       # 
#       keys2source = response4delete2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4delete2programwithbinary ['CODE4BINARY'] [key2source] [1])
#       # COUNT2REFERENCE: 1
#       # COUNT4BINARY: 1
#       # DIRECTORY4BINARY: ...//__pycache__//
#       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000003B46AC8>
#       # LANGUAGE4BINARY: Python
#       # SIZE4BINARY: {'...//__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
#       # BINARY:  ... //__pycache__//dpu2image4xi.cpython-33.pyc                       
#   except:
#       print('(Step 3Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithbinary : ', response4delete2programwithbinary)   
#          
#">

# -------------------------------------------------------------------------------------------------   
def delete2programwithbinary4dpu(task4program2dpu):

    '''
    deletes program for DPU
    '''
    global dpu2program4xi                                                                                     


    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2programwithbinary4dpu" 
    method2print = "delete2programwithbinary4dpu"

    message2print = "{(Step 10A/70) started" + "; delete2programwithbinary4dpu-started: " + str(delete2programwithbinary4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2programwithbinary4dpu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2dpu-to-delete2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2programbinary = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS BINARY KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4binary = task4program2dpu ['CODE4BINARY'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4binary-using-by-delete2programwithbinary4dpu: " + str(sorted(keys2delete4binary))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE BINARY INSIDE PROGRAM OBJECT FOR DPU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2programwithbinary4dpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4binary in keys2delete4binary:
            try:            
                del dpu2program4xi ['CODE4BINARY'] [key2delete4binary]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4binary-deleted-by-delete2programwithbinary4dpu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20C-' 
                
                del dpu2program4xi ['SIZE4BINARY'] [key2delete4binary]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4binary-deleted-by-delete2programwithbinary4dpu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20D-' 
                
                dpu2program4xi ['COUNT4BINARY'] = dpu2program4xi ['COUNT4BINARY'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4binary-upgraded-by-delete2programwithbinary4dpu: " + str(dpu2program4xi ['COUNT4BINARY'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #                                
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4binary-yielded-by-delete2programwithbinary4dpu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4program2dpu-dpu2program4xi-failed-by-delete2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2binaryprograms = dpu2program4xi ['CODE4BINARY'] .keys()
        for key2binaryprograms in keys2binaryprograms:
            message2print = "{(Step 70A/70) got" + "; key2binaryprograms-got-by-delete2programwithbinary4dpu: " + str(key2binaryprograms) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #  
    except:
        pass 
               
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4dpu-returning-by-delete2programwithbinary4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 


    return dpu2program4xi
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2programwithsource4dpu" usage = "
#     
#   print('(Step 2A4/700) OF DPU2PROGRAM4XI5TEST - TRYING TO DELETE SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2program = {}
#       task4delete2program ['CODE4SOURCE'] = {}
#       task4delete2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = 'Yes'
#       task4delete2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2dummy4xi.py'] = 'Yes'
#            task4delete2program ['SIZE4SOURCE'] = {}
#       task4delete2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = 'Yes'
#       task4delete2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2dummy4xi.py'] = 'Yes'
#       
#       response4delete2programwithsource = delete2programwithsource4dpu(task4delete2program)
#       print('(Step 2A5/700) OF DPU2PROGRAM4XI5TEST - DELETED SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 2Z4/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO DELETE SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4delete2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4delete2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4delete2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4delete2programwithsource['SIZE4SOURCE'])
#       keys2source = response4delete2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4delete2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           # COUNT4SOURCE:  1
#           # DIRECTORY4SOURCE:  OK
#           # LANGUAGE4SOURCE:  Python
#           # SIZE4SOURCE:  {'... //dpu2sampler4xi5test.py': 1337}
#           # SOURCE:  ... //dpu2sampler4xi5test.py
#           # ... profile = "11" />       
#   except:
#       print('(Step 2Z5/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithsource *** ', response4delete2programwithsource) 
#          
#">

# -------------------------------------------------------------------------------------------------   
def delete2programwithsource4dpu(task4program2dpu):

    '''
    deletes program for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    

    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2programwithsource4dpu" 
    method2print = "delete2programwithsource4dpu"

    message2print = "{(Step 10A/70) started" + "; delete2programwithsource4dpu-started: " + str(delete2programwithsource4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2programwithsource4dpu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2dpu-to-delete2programwithsource4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 'Yes'}, 
    #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 'Yes'}
    # }; 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2programsource = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS SOURCE KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4source = task4program2dpu ['CODE4SOURCE'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4source-using-by-delete2programwithsource4dpu: " + str(sorted(keys2delete4source))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # [folder2lib4dpu + 'dpu2buffer4xi.py']
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE SOURCE INSIDE PROGRAM OBJECT FOR DPU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2programwithsource4dpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4source in keys2delete4source:
            try:            
                del dpu2program4xi ['CODE4SOURCE'] [key2delete4source]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4source-deleted-by-delete2programwithsource4dpu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #folder2lib4dpu +  dpu2buffer4xi.py; dpu2dummy4xi5test.py
                
                code2ack  = code2ack + '20C-' 
                
                del dpu2program4xi ['SIZE4SOURCE'] [key2delete4source]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4source-deleted-by-delete2programwithsource4dpu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #folder2lib4dpu +  dpu2buffer4xi.py
                
                code2ack  = code2ack + '20D-' 
                
                dpu2program4xi ['COUNT4SOURCE'] = dpu2program4xi ['COUNT4SOURCE'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4source-upgraded-by-delete2programwithsource4dpu: " + str(dpu2program4xi ['COUNT4SOURCE'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 1                                  
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4source-yielded-by-delete2programwithsource4dpu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #folder2lib4dpu +  dpu2dummy4xi.py                    
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4program2dpu-dpu2program4xi-failed-by-delete2programwithsource4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2sourceprograms = dpu2program4xi ['CODE4SOURCE'] .keys()
        for key2sourceprograms in keys2sourceprograms:
            message2print = "{(Step 70A/70) got" + "; key2sourceprograms-got-by-delete2programwithsource4dpu: " + str(key2sourceprograms) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #folder2lib4dpu +  dpu2sampler4xi5test.py    
    except:
        pass 
               
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4dpu-returning-by-delete2programwithsource4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...

    return dpu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2programwithbinary4dpu" usage = "
#     
#   print('(Step 3A1/700) OF DPU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM BINARY FOR DPU ', time4xi())          
#   try:
#       response4create2programwithbinary = create2programwithbinary4dpu()
#       print('(Step 3A2/700) OF DPU2PROGRAM4XI5TEST - CREATED PROGRAM BINARY FOR DPU ', time4xi())
#   except:
#       print('(Step 3Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM BINARY FOR DPU *** ', time4xi())
#   try:                                                                                          
#       keys2response = response4create2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4create2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4create2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4create2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4create2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4create2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4create2programwithbinary ['SIZE4BINARY'])                
#       # COUNT2REFERENCE: 1
#       # COUNT4BINARY: 1
#       # DIRECTORY4BINARY: ...__pycache__//
#       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000003F53AC8>
#       # LANGUAGE4BINARY: Python
#       # SIZE4BINARY: {'...__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
#       keys2source = response4create2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4create2programwithbinary ['CODE4BINARY'] [key2source] [1])
#           # BINARY: ...//__pycache__//dpu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'    
#   except:
#       print('(Step 3Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithbinary : ', response4create2programwithbinary)
#
#">

# -------------------------------------------------------------------------------------------------   
def create2programwithbinary4dpu(task4program2dpu=None):

    '''
    creates program with binary for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    

    global dir2binary4dpu                                                                                           # Directory for binary files      

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2programwithbinary4dpu" 
    method2print = "create2programwithbinary4dpu"

    message2print = "{(Step 1A/70) started" + "; create2programwithbinary4dpu-started: " + str(create2programwithbinary4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2programwithbinary4dpu at 0x0000000003B19048>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2dpu-to-create2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    dpu2program4xi = {}                                                                                             # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1E/70) initialized" + "; dpu2program4xi-initialized-by-create2programwithbinary4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                 
    # {}
    try:              
        if task4program2dpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO DPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4program2dpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4program2dpu-created-by-create2programwithbinary4dpu: " + str(task4program2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4dpu = trace2program4dpu                                                               # CALLBACK FUNCTION FOR DPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4dpu-created-by-create2programwithbinary4dpu: " + str(function2callback4dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2program4dpu at 0x00000000039F12C8>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2programwithbinary4dpu"                                               # MODULE OF DPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2programwithbinary4dpu"                                               # FUNCTION OF DPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2programwithbinary4dpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR DPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2programwithbinary4dpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4dpu-failed ; time: 22:50:41 11/25/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithbinary4dpu', 
            #  'METHOD': 'create2programwithbinary4dpu'}; time: 22:50:41 11/25/11 Pacific Standard Time}


            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4program2dpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO DPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4program2dpu['FUNCTION2CALLBACK'] = function2callback4dpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4program2dpu-created-by-create2programwithbinary4dpu: " + str(task4program2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2program4dpu at 0x0000000003B18AC8>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4dpu-failed ; time: 22:50:41 11/25/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithbinary4dpu', 'METHOD': 'create2programwithbinary4dpu'}}; time: 22:50:41 11/25/11 Pacific Standard Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2programwithbinary4dpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2programwithbinary4dpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 22:50:41 11/25/11 Pacific Standard Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2programwithbinary4dpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4program2dpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2dpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2dpu-trying-by-create2programwithbinary4dpu: " + str(context2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2dpu = get2context4dpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2dpu-got-by-create2programwithbinary4dpu: " + str(context2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK':  ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2dpu-failed-by-create2programwithbinary4dpu: " + str(context2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4program2dpu ['CONTEXT'] = context2dpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2dpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2dpu-trying-by-create2programwithbinary4dpu: " + str(device2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2dpu = get2dpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2dpu-got-by-create2programwithbinary4dpu: " + str(device2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2dpu-failed-by-create2programwithbinary4dpu: " + str(device2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4program2dpu ['DEVICE'] = device2dpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4program2dpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2dpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2dpu-trying-by-create2programwithbinary4dpu: " + str(queue2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2dpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2dpu-got-by-create2programwithbinary4dpu: " + str(queue2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003BD1E10>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2dpu-failed-by-create2programwithbinary4dpu: " + str(queue2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4program2dpu ['QUEUE'] = queue2dpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2dpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2dpu-trying-by-create2programwithbinary4dpu: " + str(pipe2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2dpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2dpu-got-by-create2programwithbinary4dpu: " + str(pipe2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2dpu-failed-by-create2programwithbinary4dpu: " + str(pipe2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4program2dpu ['PIPE'] = pipe2dpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2dpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2dpu-trying-by-create2programwithbinary4dpu: " + str(commandqueueinfo2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2dpu = get2commandqueueinfo4dpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2dpu-got-by-create2programwithbinary4dpu: " + str(commandqueueinfo2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2dpu-failed-by-create2programwithbinary4dpu: " + str(commandqueueinfo2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4program2dpu ['COMMANDQUEUE'] = commandqueueinfo2dpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2dpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2dpu-trying-by-create2programwithbinary4dpu: " + str(bufferinfo2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2dpu = get2bufferinfo4dpu()                                                                   # SYSTEM CREATES BUFFER FOR DPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2dpu-got-by-create2programwithbinary4dpu: " + str(bufferinfo2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2dpu-failed-by-create2programwithbinary4dpu: " + str(bufferinfo2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4program2dpu ['BUFFER'] = bufferinfo2dpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2dpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2dpu-trying-by-create2programwithbinary4dpu: " + str(eventinfo2dpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2dpu = get2eventinfo4dpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2dpu-got-by-create2programwithbinary4dpu: " + str(eventinfo2dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2dpu-failed-by-create2programwithbinary4dpu: " + str(eventinfo2dpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4program2dpu ['EVENT'] = eventinfo2dpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) COUNT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            task4program2dpu ['COUNT'] = 0
            message2print = "{(Step 11A/70) trying" + "; count2dpu-trying-by-create2programwithbinary4dpu: " + str(task4program2dpu['COUNT'])  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 0            
            try:            
                task4program2dpu ['COUNT'] = int(len(kb2fetch4dpu)/2)                                               # number of binary files to be read and embedded
                message2print = "{(Step 11B/70) set" + "; count2dpu-set-by-create2programwithbinary4dpu to: " + str(task4program2dpu['COUNT'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 1                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; count2dpu-failed-by-create2programwithbinary4dpu: " + str(task4program2dpu['COUNT']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
                               
            # ---------------------------------------------------------------------------------------------
            # (12) DIRECTORY - EXTRA                       
            # ---------------------------------------------------------------------------------------------             
            dir2dpu = dir2binary4dpu                                                                                # default Python cache folder for binaries                                                                                                                                                                 
            
            message2print = "{(Step 12B/70) set" + "; dir2dpu-set-by-create2programwithbinary4dpu to: " + str(dir2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4dpu +  __pycache__//
            
            code2ack  = code2ack + '12B-' 
            task4program2dpu ['DIRECTORY'] = dir2dpu
            
                        
                        
            # ---------------------------------------------------------------------------------------------
            # (14) CODE AND SIZE - REQUIRED                   
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A/70) using" + "; DirectoryWalker-using-by-create2programwithbinary4dpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # <class 'dpu2program4xi.DirectoryWalker'> 
            
            for file2walk in DirectoryWalker(dir2dpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B/70) examining" + "; directory-examining-by-create2programwithbinary4dpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                      
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C/70) examining" + "; file-examining-by-create2programwithbinary4dpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # folder2lib4dpu +  __pycache__//dpu2buffer4xi.cpython-33.pyc
                    
                    message2print = "{(Step 14D/70) calling" + "; fetch2programbinary4dpu-calling-by-create2programwithbinary4dpu: " + str(fetch2programbinary4dpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2programbinary4dpu at 0x00000000040079C8>
                    
                    message2print = "{(Step 14E/70) passing" + "; file2walk-passing-to-fetch2programbinary4dpu-by-create2programwithbinary4dpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4dpu +  __pycache__//dpu2buffer4xi.cpython-33.pyc
                    
                    fetch2programbinary4dpu(file2walk)                                                                 # fetches program files into fetch2code4binary dictionary 
                    
                    message2print = "{(Step 14F/70) performed" + "; fetch2programbinary4dpu-performed-by-create2programwithbinary4dpu: " + str(fetch2programbinary4dpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # <function fetch2programbinary4dpu at 0x00000000040079C8>
                    
            task4program2dpu ['CODE'] = fetch2code4binary        
            size2binary4dpu  = {}
            keys2binary4dpu = fetch2code4binary.keys()                                                                 # dpu2sampler4xi5test.py
            for key2binary4dpu in keys2binary4dpu:
                size2binary4dpu[key2binary4dpu] = len(fetch2code4binary[key2binary4dpu])                               # length of saved binary   
                
                message2print = "{(Step 14H/70) using" + "; key2binary4dpu-using-by-create2programwithbinary4dpu: " + str(key2binary4dpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # folder2lib4dpu +  __pycache__//dpu2image4xi.cpython-33.pyc                       
                message2print = "{(Step 14I/70) set" + "; size2binary4dpu-set-by-create2programwithbinary4dpu to: " + str(len(fetch2code4binary[key2binary4dpu]))  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 1006
            
            code2ack  = code2ack + '14B-' 
        
            task4program2dpu ['SIZE'] = size2binary4dpu
            message2print = "{(Step 14J/70) set" + "; size-of-task4program2dpu-set-by-create2programwithbinary4dpu to: " + str(task4program2dpu ['SIZE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  folder2lib4dpu +  __pycache__//dpu2image4xi.cpython-33.pyc': 1006}
                            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            language2dpu  = "Python"  # C++; C
           
            message2print = "{(Step 15B/70) set" + "; language2dpu-set-by-create2programwithbinary4dpu to: " + str(language2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '15B-' 
        
            task4program2dpu ['LANGUAGE'] = language2dpu     

                                    
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2dpu  = 1
           
            message2print = "{(Step 17B/70) set" + "; count2reference2dpu-set-by-create2programwithbinary4dpu to: " + str(count2reference2dpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '17B-' 
        
            task4program2dpu ['COUNT2REFERENCE'] = count2reference2dpu                                 
                                                                                                                      
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2dpu-yielded-by-create2programwithbinary4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4program2dpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2programwithbinary4dpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER', 'CODE' ... 
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR DPU ! 
    # ---------------------------------------------------------------------------------------------    
    dpu2program4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; dpu2program4xi-initialized-by-create2programwithbinary4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    if 'DATA2CALLBACK' in keys2task:    
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - EXTRA 
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['DATA2CALLBACK'] = task4program2dpu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4dpu-failed ; time: 02:37:08 11/17/11 Pacific Standard Time}', 'CLASS': 'create2programwithbinary4dpu', 'METHOD': 'create2programwithbinary4dpu'}; time: 02:37:08 11/17/11 Pacific Standard Time}
            
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'

    if 'FUNCTION2CALLBACK' in keys2task:       
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['FUNCTION2CALLBACK'] = task4program2dpu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2program4dpu at 0x00000000039912C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 

    if 'PLATFORM' in keys2task:           
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['PLATFORM'] = task4program2dpu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:37:08 11/17/11 Pacific Standard Time}
            
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
        
        

    if 'CONTEXT' in keys2task:         
        message2print = "{(Step 23A/70) try" + "; context2dpu-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT - REQUIRED
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['CONTEXT'] = task4program2dpu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...       
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 

    if 'DEVICE' in keys2task:           
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['DEVICE'] = task4program2dpu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'RESOLUTION2TIMER4PROFILING': 318 ... 
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'

    if 'CODE2ERROR' in keys2task:                    
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['CODE2ERROR'] = task4program2dpu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'                
                

    if 'QUEUE' in keys2task:             
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['QUEUE'] = task4program2dpu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003BEE630>
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z1/70) yielded" + "; queue-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-' 

    if 'PIPE' in keys2task:                                   
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - EXTRA
            # -----------------------------------------------------------------------------------------               
            dpu2program4xi['PIPE'] = task4program2dpu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-dpu2program4xi-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'   

    if 'COMMANDQUEUE' in keys2task:   
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['COMMANDQUEUE']  = task4program2dpu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    

    if 'BUFFER' in keys2task:   
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['BUFFER']  = task4program2dpu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            

    if 'EVENT' in keys2task:   
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['EVENT']  = task4program2dpu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'            
                

    if 'COUNT4BINARY' in keys2task:   
        message2print = "{(Step 31A/70) try" + "; count-try-by-create2programwithbinary4dpu: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['COUNT4BINARY']  = task4program2dpu['COUNT']  
     
            message2print = "{(Step 31B/70) initialized" + "; count-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '31B-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     

    if 'DIRECTORY4BINARY' in keys2task:   
        message2print = "{(Step 32A/70) try" + "; DIRECTORY-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['DIRECTORY4BINARY']  = task4program2dpu['DIRECTORY']  
     
            message2print = "{(Step 32B/70) initialized" + "; directory-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['DIRECTORY4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #folder2lib4dpu +  __pycache__//
            code2ack  = code2ack + '32B-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; DIRECTORY-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'  

    if 'CODE4BINARY' in keys2task:               
        message2print = "{(Step 33A/70) try" + "; binary-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)        
        dpu2program4xi['CODE4BINARY'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['CODE4BINARY'] = task4program2dpu ['CODE']
    
     
            message2print = "{(Step 33B/70) created" + "; code-created-by-create2programwithsource4dpu: " + str(len(dpu2program4xi['CODE4BINARY']))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
             
            code2ack  = code2ack + '33B-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; binary-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '33Z1-'     

    if 'SIZE4BINARY' in keys2task:           
        message2print = "{(Step 34A/70) try" + "; size-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            dpu2program4xi['SIZE4BINARY']  = task4program2dpu['SIZE']  
     
            message2print = "{(Step 34B/70) initialized" + "; size-initialized-by-create2programwithbinary4dpu to: " + str(dpu2program4xi['SIZE4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
            code2ack  = code2ack + '34B-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     

    if 'LANGUAGE4BINARY' in keys2task:           
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['LANGUAGE4BINARY']  = task4program2dpu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['LANGUAGE4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'     

    if 'COUNT2REFERENCE' in keys2task:                   
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            dpu2program4xi['COUNT2REFERENCE']  = task4program2dpu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2programwithbinary4dpu: " + str(dpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-'     
        
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-create2programwithbinary4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ... CODE4BINARY ...
    
    return dpu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2programsource4dpu" usage = "
#   fetch2programsource4dpu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2programsource4dpu(program2explore4dpu):
    global fetch2code4source                                                                                        # Fetched code dictionary
    
    global kb2fetch4dpu                                                                                             # Knowledgebase for fetching
    global profile2fetch4source                                                                                     # Extension for files
    global keys2profile4source                                                                                      
    

    class2print  = "fetch2programsource4dpu"
    method2print = "fetch2programsource4dpu" 
     
    message2print = "{(Step 0A/33) passed" + "; program2explore4dpu-passed-to-fetch2programsource4dpu: " + str(program2explore4dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    #folder2lib4dpu +  dpu2buffer4xi.py; dpu2dummy4xi5test.cpp
    
    message2print = "{(Step 0B/33) using" + "; keys2profile4source-using-by-fetch2programsource4dpu: " + str(keys2profile4source)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # ['.cl', '.c', '.py', '.cpp']
                  
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4source=os.path.splitext(program2explore4dpu)

        
    if name2program4source[1] in keys2profile4source:
        try:          
            # -----------------------------------------------------------------------------------------
            # (Step 1/33) OF FETCH2SCOPE4XI - SOURCE PROGRAM
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 1A/33) analyzing" + "; program2explore4dpu-analyzing-by-fetch2programsource4dpu: " + str(program2explore4dpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            #folder2lib4dpu +  dpu2buffer4xi.py
                 
            path2input4source = program2explore4dpu
            message2print = "{(Step 1B/33) set" + "; path2input4source-set-by-fetch2programsource4dpu to: " + str(path2input4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #folder2lib4dpu +  dpu2buffer4xi.py 
    
            # -----------------------------------------------------------------------------------------
            # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2A/33) analyzing" + "; path2input4source-analyzing-by-fetch2programsource4dpu: " + str(path2input4source) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #folder2lib4dpu +  dpu2buffer4xi.py; dpu2dummy4xi5test.cpp
                
            keys2fetch4source = kb2fetch4dpu.keys()
            ack2match4source = False
            for key2fecth4source in keys2fetch4source:                                                              # 1,2 ...
                # -----------------------------------------------------------------------------------------
                # 
                # -----------------------------------------------------------------------------------------
                index2search=str.find(path2input4source,kb2fetch4dpu[key2fecth4source])                             # kb2fetch4dpu[1] = folder2lib4dpu + 'dpu2sampler4xi5test.py'
                if index2search == -1:                                         
                    # -----------------------------------------------------------------------------------------
                    # VALID FILE TYPE BUT NOT SELECTED TO BE FETCHED - NOT BE SKIPPED MIGHT GO THROUGH HERE TOO
                    # -----------------------------------------------------------------------------------------               
                    message2print = "{(Step 1E/33) skipped" + "; path2input4source-skipped-by-fetch2programsource4dpu: " + str(path2input4source)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                      
                    #folder2lib4dpu +  dpu2buffer4xi.py 
    
                else:
                    ack2match4source = True
                    
                    message2print = "{(Step 2B/33) decided" + "; program-to-read-decided-by-fetch2programsource4dpu: " + str(path2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)        
                    #folder2lib4dpu +  dpu2sampler4xi5test.py; dpu2dummy4xi5test.cpp
    
                    # -----------------------------------------------------------------------------------------
                    # (Step 3/33) OF FETCH2SCOPE4XI - OPEN INPUT
                    # -----------------------------------------------------------------------------------------
                    file2input4source = open(path2input4source,'r')
                    message2print = "{(Step 3A/33) opened" + "; file2input4source-opened-by-fetch2programsource4dpu: " + str(file2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)     
                    # <_io.TextIOWrapper name=folder2lib4dpu + 'dpu2sampler4xi5test.py' mode='r' encoding='cp1252'> ; dpu2dummy4xi5test.cpp 
                    
                    # -----------------------------------------------------------------------------------------
                    # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT 
                    # -----------------------------------------------------------------------------------------
                    key2save4source = path2input4source
                    fetch2code4source[key2save4source] = {}                                                                # fetch2code4source['dpu2sampler4xi5test.py'] = {}
                    number2line4source = 0  
                    message2print = "{(Step 3B/33) saving" + "; key2fecth4source-saving-by-fetch2programsource4dpu: " + str(key2fecth4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # 
             
                    while 1:
                        # -------------------------------------------------------------------------------------
                        # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                        # -------------------------------------------------------------------------------------             
                        line2process4source = file2input4source.readline()  
                        if not line2process4source:
                            # ---------------------------------------------------------------------------------
                            # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                            # ---------------------------------------------------------------------------------             
                            break                                               
                        else:
                            # ---------------------------------------------------------------------------------
                            # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                            # ---------------------------------------------------------------------------------
                            message2print = "{(Step 7A/33) read" + "; line2process4source-read-by-fetch2programsource4dpu: " + str(line2process4source) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)              
                            # ... profile = "11" />
                            # ...
                            # # end of dpu2sampler4xi5test.py 
                            # #/> 
                            try:
                                number2line4source = number2line4source + 1
                                fetch2code4source[key2save4source][number2line4source] = line2process4source
                            except:
                                message2print = "{(Step 7Z1/33) failed" + "; key2fecth4source-failed-by-fetch2programsource4dpu: " + str(key2fecth4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z2/33) failed" + "; line2process4source-failed-by-fetch2programsource4dpu: " + str(line2process4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z3/33) failed" + "; number2line4source-failed-by-fetch2programsource4dpu: " + str(number2line4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)                            
                                                                 
                    # -----------------------------------------------------------------------------------------
                    # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                    # -----------------------------------------------------------------------------------------            
                    file2input4source.close()
                    message2print = "{(Step 31A/33) closed" + "; file2input4source-closed-by-fetch2programsource4dpu: " + str(file2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)           
                    # <_io.TextIOWrapper name=folder2lib4dpu + 'dpu2sampler4xi5test.py' mode='r' encoding='cp1252'>; dpu2dummy4xi5test.cpp
                        
            
                    message2print = "{(Step 31B/33) saved" + "; fetch2code4source-saved-by-fetch2programsource4dpu: " + str(fetch2code4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # {folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: '# -- ....
                     
                    # ---------------------------------------------------------------------------------------------
                    #
                    # ---------------------------------------------------------------------------------------------                                          
        except:
            message2print = "{(Step 1Z1/33) failed" + "; name2program4source-failed-by-fetch2programsource4dpu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
    else:
        try:
            # -----------------------------------------------------------------------------------------
            # INVALID FILE TYPE - NOT IN PROFILE
            # -----------------------------------------------------------------------------------------       
            message2print = "{(Step 1F/33) skipped" + "; name2program4source-skipped-by-fetch2programsource4dpu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            # folder2lib4dpu + '__pycache__//dpu2buffer4xi.cpython-32', '.pyc')
            
            extension2skip = name2program4source[1]
            message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2programsource4dpu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # pyc
    
            message2print = "{(Step 1H/33) returning" + "; fetch2programsource4dpu-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)   
        except:
            message2print = "{(Step 1Z2/33) yielded" + "; name2program4source-yielded-by-fetch2programsource4dpu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
    message2print = "{(Step 1H/33) returning" + "; fetch2programsource4dpu-returning: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)            
    #   
                                            
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programwithsource4dpu" usage = "
#
#   ... prepare task4set2programws ...    
#
#   response4set2program = {} 
#   print('(Step 17F/700) OF DPU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM WITH SOURCE: ', set2programwithsource4dpu)   
#   # <function set2programwithsource4dpu at 0x0000000003837948>
#   try:                               
#       response4set2pws = set2programwithsource4dpu(task4set2programws)
#       print('(Step 17G/700) OF DPU2PROGRAM4XI5TEST - SET PROGRAM SOURCE WITH SOURCE: ', set2programwithsource4dpu)                                                                                                           
#   except:
#       print('(Step 17Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM SOURCE WITH SOURCE: ', set2programwithsource4dpu)  
#   try:                                                                                          
#       keys2response = response4set2pws.keys()
#       print('COUNT2REFERENCE:', response4set2pws ['COUNT2REFERENCE'])   
#       print('COUNT4SOURCE:', response4set2pws ['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE:', response4set2pws ['DIRECTORY4SOURCE']) 
#       print('FUNCTION2CALLBACK:', response4set2pws ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4SOURCE:', response4set2pws ['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE:', response4set2pws ['SIZE4SOURCE'])                
#        keys2source = response4set2pws['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4set2pws ['CODE4SOURCE'] [key2source] [1])
#           #
#           # COUNT2REFERENCE: 1
#           # COUNT4SOURCE: 1
#           # DIRECTORY4SOURCE: OK
#           # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000003A28B48>
#           # LANGUAGE4SOURCE: Python
#           # SIZE4SOURCE: 1
#           # SOURCE:  ... dpu2image4xi5test.py
#           # ... profile = "11" />                  
#   except:
#       print('(Step 3Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4set2pws : ', response4set2pws)                                                   
#
#">

# -------------------------------------------------------------------------------------------------   
def set2programwithsource4dpu(task4programset2dpu=None):

    '''
    sets programs for DPU
    '''
    global dpu2program4xi  

    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2programwithsource4dpu"
    method2print = "set2programwithsource4dpu" 

    message2print = "{(Step 1A/33) started" + "; set2programwithsource4dpu-started: " + str(set2programwithsource4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2programwithsource4dpu at 0x0000000003837948> 
    
    keys2task = task4programset2dpu.keys()
    for key2task in sorted(keys2task):
        message2print = "{(Step 1B/33) passed" + "; key2task-passed-to-set2programwithsource4dpu: " + str(key2task) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # CODE2ERROR
        # CONTEXT 
        # COUNT4SOURCE
        # DEVICE
        # DIRECTORY4SOURCE
        # LANGUAGE4SOURCE
        # PROPERTIES
        # QUEUE
          
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dpu2program4xi is None:
            dpu2program4xi = create2programwithsource4dpu()                                                                              # INITIALIZE PROGRAM FOR DPU 
            message2print = "{(Step 1C/70) set" + "; dpu2program4xi-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2programwithsource4dpu-skipped-by-set2programwithsource4dpu: " + str(create2programwithsource4dpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            dpu2program4xi = create2programwithsource4dpu()
            message2print = "{(Step 1E/70) set" + "; dpu2program4xi-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dpu2program4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET       
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['DATA2CALLBACK'] = task4programset2dpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['FUNCTION2CALLBACK'] = task4programset2dpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['PLATFORM'] = task4programset2dpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2dpu-try-by-set2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['CONTEXT'] = task4programset2dpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['DEVICE'] = task4programset2dpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['CODE2ERROR'] = task4programset2dpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['QUEUE'] = task4programset2dpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['PIPE'] = task4programset2dpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-dpu2program4xi-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['COMMANDQUEUE']  = task4programset2dpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
        dpu2program4xi['BUFFER']  = task4programset2dpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2programwithsource4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['EVENT']  = task4programset2dpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31)  - COUNT                           
        # ---------------------------------------------------------------------------------------------             
        dpu2program4xi['COUNT4SOURCE']  = task4programset2dpu['COUNT4SOURCE']  
 
        message2print = "{(Step 31B/70) set" + "; count-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'  

    message2print = "{(Step 32A/70) try" + "; directory-try-by-set2programwithsource4dpu: " + str(task4programset2dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print) 
    # {'CODE2ERROR': 0 ... 'DIRECTORY4SOURCE': '...//a' ... 

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        dpu2program4xi['DIRECTORY4SOURCE']  = task4programset2dpu['DIRECTORY4SOURCE'] 
        
        message2print = "{(Step 32B/70) set" + "; directory-set-by-set2programwithdirectory4dpu to: " + str(dpu2program4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # OK
        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory-yielded-by-set2programwithdirectory4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-' 
        
    message2print = "{(Step 33A/70) try" + "; code-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['CODE4SOURCE']  = task4programset2dpu['CODE4SOURCE']  
 
        message2print = "{(Step 33B/70) set" + "; code-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4dpu + 'dpu2image4xi5test.py': 
        #  {1: '... profile = "11" />\n', 
        #   2: '... program = "dpu2image4xi5test.py">\n'}
 
        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; code-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-'             
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['SIZE4SOURCE']  = task4programset2dpu['SIZE4SOURCE']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2programwithsource4dpu: " + str(task4programset2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['LANGUAGE4SOURCE']  = task4programset2dpu['LANGUAGE4SOURCE']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'          
                        
    message2print = "{(Step 36A/70) try" + "; count2reference-try-by-set2programwithsource4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['COUNT2REFERENCE']  = task4programset2dpu['COUNT2REFERENCE']  
 
        message2print = "{(Step 36B/70) set" + "; count2reference-set-by-set2programwithsource4dpu to: " + str(dpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-set2programwithsource4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '36Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-set2programwithsource4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE ...

    return dpu2program4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2programbinary4dpu" usage = "
#   fetch2programbinary4dpu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2programbinary4dpu(program2explore4dpu):
    global fetch2code4binary                                                                                        # Fetched code dictionary
    
    global kb2fetch4dpu                                                                                             # Knowledgebase for fetching

    global keys2profile4binary         

        
    class2print  = "fetch2programbinary4dpu"
    method2print = "fetch2programbinary4dpu" 
 
    message2print = "{(Step 1/8) got" + "; program2explore4dpu-got-by-fetch2programbinary4dpu: " + str(program2explore4dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #folder2lib4dpu +  __pycache__//dpu2buffer4xi.cpython-33.pyc
               
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4binary=os.path.splitext(program2explore4dpu)
    
    if name2program4binary[1] in keys2profile4binary:          
        # -----------------------------------------------------------------------------------------
        # (Step 1/33) OF FETCH2SCOPE4XI - PYTHON BINARY
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 1A/33) analyzing" + "; program2explore4dpu-analyzing-by-fetch2programbinary4dpu: " + str(program2explore4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        #folder2lib4dpu +  __pycache__//dpu2buffer4xi.cpython-33.pyc 
        
        path2input4binary = program2explore4dpu                                                                          
        message2print = "{(Step 1B/33) set" + "; path2input4binary-set-by-fetch2programbinary4dpu to: " + str(path2input4binary)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        #folder2lib4dpu +  __pycache__//dpu2buffer4xi.cpython-33.pyc
        
        # 
        # -----------------------------------------------------------------------------------------
        # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED 
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A1/33) analyzing" + "; path2input4binary-analyzing-by-fetch2programbinary4dpu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #folder2lib4dpu +  __pycache__//dpu2buffer4xi.cpython-33.pyc 
        # ...
        #folder2lib4dpu +  __pycache__//dpu2sampler4xi.cpython-33.pyc
        keys2fetch4binary = kb2fetch4dpu.keys()
        ack2match4binary = False
        for key2fecth4binary in keys2fetch4binary:                                                                  # 
            string2search4binary = kb2fetch4dpu[key2fecth4binary]                                                   # dpu2sampler4xi.pyc
            index2search=str.find(path2input4binary,string2search4binary)                                           # E:\Scope4xi\041721\zap\lib\a\__pycache__\dpu2sampler4xi.cpython-33.pyc
            if index2search == -1:                                         
                pass                                                                                                # not ordered to be fetched
            else:
                ack2match4binary = True                                                                             # E:\Scope4xi\041721\zap\lib\a\__pycache__\dpu2sampler4xi.cpython-33.pyc
                    
        if ack2match4binary is True:
            message2print = "{(Step 2B/33) decided" + "; program-to-read-decided-by-fetch2programbinary4dpu: " + str(path2input4binary) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)        
            # folder2lib4dpu +  __pycache__//dpu2image4xi.cpython-33.pyc


            message2print = "{(Step 2B/33) opening" + "; path2input4binary-opening-by-fetch2programbinary4dpu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            # folder2lib4dpu +  __pycache__//dpu2image4xi.cpython-33.pyc 
            try:
                # -----------------------------------------------------------------------------------------
                # (Step 3/33) OF FETCH2SCOPE4XI - OPEN BINARY INPUT
                # -----------------------------------------------------------------------------------------
                file2input4binary = open(path2input4binary,'rb')
                message2print = "{(Step 3A/33) opened" + "; file2input4binary-opened-by-fetch2programbinary4dpu: " + str(file2input4binary) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)     
                # <_io.BufferedReader name=folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'>
            
                try:
                    # -----------------------------------------------------------------------------------------
                    # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT
                    # -----------------------------------------------------------------------------------------
                    key2save4binary = path2input4binary
                    fetch2code4binary[key2save4binary] = {}                                                                # 
                    number2line4binary = 0  
                    message2print = "{(Step 3B/33) saving" + "; key2save4binary-saving-by-fetch2programbinary4dpu: " + str(key2save4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # folder2lib4dpu +  __pycache__//dpu2sampler4xi.cpython-33.pyc
        
                    while 1:
                        # -------------------------------------------------------------------------------------
                        # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                        # -------------------------------------------------------------------------------------             
                        line2process4binary = file2input4binary.readline()  
                        if not line2process4binary:
                            # ---------------------------------------------------------------------------------
                            # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                            # ---------------------------------------------------------------------------------             
                            break                                               
                        else:
                            try:
                                # ---------------------------------------------------------------------------------
                                # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                                # ---------------------------------------------------------------------------------
                                number2line4binary = number2line4binary + 1
                                # message2print = "{(Step 7A/33) reading" + "; number2line4binary-reading-by-fetch2programbinary4dpu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                                # print4xi(class2print, method2print, message2print)              
                                # 1,2,3 .... 1006
                                                
                                fetch2code4binary[key2save4binary][number2line4binary] = line2process4binary
                            except:
                                message2print = "{(Step 7Z1/33) failed" + "; key2fecth4binary-failed-by-fetch2programbinary4dpu: " + str(key2fecth4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z2/33) failed" + "; line2process4binary-failed-by-fetch2programbinary4dpu: " + str(line2process4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z3/33) failed" + "; number2line4binary-failed-by-fetch2programbinary4dpu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)        
                except:        
                    message2print = "{(Step 7Z4/33) failed" + "; number2line4binary-failed-by-fetch2programbinary4dpu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 52           
                finally:
                    # -----------------------------------------------------------------------------------------
                    # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                    # -----------------------------------------------------------------------------------------            
                    file2input4binary.close()
                    message2print = "{(Step 31A/33) closed" + "; file2input4binary-closed-by-fetch2programbinary4dpu: " + str(file2input4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)           
                    # <_io.BufferedReader name=folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'>
            except:
                message2print = "{(Step 7Z4/33) failed" + "; opening-failed-by-fetch2programbinary4dpu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  

        else:
            pass
    else:
        # -----------------------------------------------------------------------------------------
        # INVALID TYPE - NOT IN PROFILE
        # -----------------------------------------------------------------------------------------       
        message2print = "{(Step 1F/33) skipped" + "; name2program4binary-skipped-by-fetch2programbinary4dpu: " + str(name2program4binary)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        # 
        
        extension2skip = name2program4binary[1]
        message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2programbinary4dpu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 

        
    message2print = "{(Step 33/33) returning" + "; fetch2programbinary4dpu-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 02:24:36 11/17/11 Pacific Standard Time
                             
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programwithbinary4dpu" usage = "
#    
#    print('(Step 4B/33) OF DPU2PROGRAM4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#    platform2xi = {}
#    try:
#        platform2xi = get2platformids4xi()
#        print('(Step 4B/33) of CONTEXT4DPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#    except:
#        print('(Step 4Z/33) of CONTEXT4DPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#    
# 
#    ... other data for set ...
#    
#    response4set2program = {} 
#    print('(Step 37F/700) OF DPU2PROGRAM4XI5TEST - TRYING TO PERFORM SET2PROGRAM4DPU: ', set2programwithbinary4dpu)   
#    # 
#    try:                               
#        response4set2program = set2programwithbinary4dpu(task4set2programwithbinary)                                                                                                         
#
#        keys2response = response4set2program.keys()
#        for key2response in sorted(keys2response):
#            print(key2response, ' : ', response4set2program[key2response])
#            # CODE4BINARY  :  {folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}
#            # CODE4SOURCE  :  {folder2lib4dpu + 'dpu2image4xi5test.py': {1: '... profile = "11" />\n', 2: '... program = "dpu2image4xi5test.py">\n'}}
#            # COUNT4BINARY  :  1
#            # COUNT4SOURCE  :  1
#            # LANGUAGE4BINARY  :  Python
#            # LANGUAGE4SOURCE  :  Python
#            # SIZE4BINARY  :  {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
#                          
#    except:
#        print('(Step 37Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PERFORM SET2PROGRAM4DPU: ', set2programwithbinary4dpu)                                            
#
#">

# -------------------------------------------------------------------------------------------------   
def set2programwithbinary4dpu(task4programsetwb2dpu=None):

    '''
    sets programs for DPU
    '''
    global dpu2program4xi  
    
      
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2programwithbinary4dpu"
    method2print = "set2programwithbinary4dpu"

    message2print = "{(Step 1A/33) started" + "; set2programwithbinary4dpu-started: " + str(set2programwithbinary4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2programwithbinary4dpu at 0x0000000003B43AC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4programsetwb2dpu-to-set2programwithbinary4dpu: " + str(task4programsetwb2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0, 
    #  'DIRECTORY4BINARY': ...//__pycache__//', 
    #  'LANGUAGE4BINARY': 'Python', 
    #  'SIZE4BINARY': 1, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003B19048>, 
    #  'CODE4BINARY': {'...//__pycache__//dpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}, 
    #   ...


    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if dpu2program4xi is None:
            dpu2program4xi = create2programwithbinary4dpu()                                                                              # INITIALIZE PROGRAM FOR DPU 
            message2print = "{(Step 1C/70) set" + "; dpu2program4xi-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2programwithbinary4dpu-skipped-by-set2programwithbinary4dpu: " + str(create2programwithbinary4dpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            dpu2program4xi = create2programwithbinary4dpu()
            message2print = "{(Step 1E/70) set" + "; dpu2program4xi-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            dpu2program4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['DATA2CALLBACK'] = task4programsetwb2dpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['FUNCTION2CALLBACK'] = task4programsetwb2dpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['PLATFORM'] = task4programsetwb2dpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2dpu-try-by-set2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['CONTEXT'] = task4programsetwb2dpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['DEVICE'] = task4programsetwb2dpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318 ... 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['CODE2ERROR'] = task4programsetwb2dpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['QUEUE'] = task4programsetwb2dpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003DF3668>
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['PIPE'] = task4programsetwb2dpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-dpu2program4xi-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['COMMANDQUEUE']  = task4programsetwb2dpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['BUFFER']  = task4programsetwb2dpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2programwithbinary4dpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['EVENT']  = task4programsetwb2dpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) COUNT - REQUIRED                           
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['COUNT4BINARY']  = task4programsetwb2dpu['COUNT4BINARY']  
 
        message2print = "{(Step 31B/70) set" + "; count-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'
          
  
    message2print = "{(Step 32A/70) try" + "; directory2binary-try-by-set2programwithdirectory2binary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY4BINARY - REQUIRED                               
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['DIRECTORY4BINARY']  = task4programsetwb2dpu['DIRECTORY4BINARY']  
 
        message2print = "{(Step 32B/70) set" + "; directory2binary-set-by-set2programwithdirectory2binary4dpu to: " + str(dpu2program4xi['DIRECTORY4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # ...//__pycache__//

        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory2binary-yielded-by-set2programwithdirectory2binary4dpu: " + str(directory2binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-'     
    
    message2print = "{(Step 33A/70) try" + "; binary-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['CODE4BINARY']  = task4programsetwb2dpu['CODE4BINARY']  
 
        message2print = "{(Step 33B/70) set" + "; binary-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['CODE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}

        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; binary-yielded-by-set2programwithbinary4dpu: " + str(binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-' 
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['SIZE4BINARY']  = task4programsetwb2dpu['SIZE4BINARY']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['SIZE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'             
        
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['LANGUAGE4BINARY']  = task4programsetwb2dpu['LANGUAGE4BINARY']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['LANGUAGE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # Python
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; count2reference-try-by-set2programwithbinary4dpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        dpu2program4xi['COUNT2REFERENCE']  = task4programsetwb2dpu['COUNT2REFERENCE']  
 
        message2print = "{(Step 35B/70) set" + "; count2reference-set-by-set2programwithbinary4dpu to: " + str(dpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; count2reference-yielded-by-set2programwithbinary4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-set2programwithbinary4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 

    return dpu2program4xi
           
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2program4dpu" usage = "
#
#   task4release2program  = {} 
#   task4release2program  ['COUNT2REFERENCE'] = 1 
#   response4release2program = {}
#   
#   print('(Step 18A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO RELEASE PROGRAM OF DPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4release2program = release2program4dpu(task4release2program)
#       print('(Step 18B/700) OF DPU2PROGRAM4XI5TEST - RELEASED PROGRAM OF DPU DEVICE ', time4xi()) 
#   except:
#       print('(Step 18Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO RELEASE PROGRAM OF DPU DEVICE ', time4xi())                                                                                                  
#   try:
#       print('(Step 18C/700) OF DPU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4release2program ['COUNT2REFERENCE'])
#       # 0
#   except:
#       print('(Step 18Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RELEASE2PROGRAM ', response4release2program)  
#   
#">

# -------------------------------------------------------------------------------------------------   
def release2program4dpu(task4release2dpu=None):

    '''
    releases dpu2program4xi
    '''
    global dpu2program4xi  
    
             
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2program4dpu"
    method2print = "release2program4dpu" 

    message2print = "{(Step 1A/33) started" + "; release2program4dpu-started: " + str(release2program4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2program4dpu at 0x0000000003A98AC8>
    
    
    message2print = "{(Step 1B/33) passed" + "; task4release2dpu-to-release2program4dpu: " + str(task4release2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    

    code2ack  = ' '
    code2nack = ' '

    try:
        if dpu2program4xi is None:
            dpu2program4xi = create2programwithsource4dpu()                                                                       # INITIALIZE PROGRAM                 
    except:
        try:
            dpu2program4xi = create2programwithsource4dpu()
        except:
            dpu2program4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; dpu2program4xi-yielded-by-release2program4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2program4dpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2dpu is None:
        task4release2dpu = {}
        task4release2dpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        dpu2program4xi['COUNT2REFERENCE'] = dpu2program4xi['COUNT2REFERENCE'] - task4release2dpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2program4dpu: " + str(dpu2program4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
         
        if dpu2program4xi['COUNT2REFERENCE'] < 0:
            dpu2program4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2program4dpu: " + " ** " + str(dpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # 
             
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            dpu2program4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2program4dpu: " + " ** " + str(dpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; dpu2program4xi-yielded-by-release2program4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-release2program4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return dpu2program4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2program4dpu" usage = "
#
#   task4retain2program  = {} 
#   task4retain2program  ['COUNT2REFERENCE'] = 1 
#   response4retain2program = {}
#   
#   print('(Step 19A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO RETAIN PROGRAM OF DPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4retain2program = retain2program4dpu(task4retain2program)
#       print('(Step 19B/700) OF DPU2PROGRAM4XI5TEST - RETAINED PROGRAM OF DPU DEVICE ', time4xi()) 
#   except:
#       print('(Step 19Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO RETAIN PROGRAM OF DPU DEVICE ', time4xi())                                                                                                  
#   try:
#       print('(Step 19C/700) OF DPU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4retain2program ['COUNT2REFERENCE'])
#       # 2
#   except:
#       print('(Step 19Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RETAIN2PROGRAM ', response4retain2program)  
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2program4dpu(task4retain2dpu=None):

    '''
    retains either dpu2program4xi
    '''
    global dpu2program4xi 
    

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2program4dpu"
    method2print = "retain2program4dpu"

    message2print = "{(Step 1A/33) started" + "; retain2program4dpu-started: " + str(retain2program4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2program4dpu at 0x0000000004095BC8>
    
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2dpu-to-retain2program4dpu: " + str(task4retain2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 
    
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2dpu is None:
        task4retain2dpu = {}
        task4retain2dpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2dpu-set-by-retain2program4dpu to: " + str(task4retain2dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass

    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2dpu.keys():
        pass
    else:
        task4retain2dpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2program4dpu to: " + str(task4retain2dpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2dpu['COUNT2REFERENCE'] < 1:
        task4retain2dpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2program4dpu to: " + str(task4retain2dpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE PROGRAM IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if dpu2program4xi is None:
            dpu2program4xi = create2programwithsource4dpu()
            message2print = "{(Step 4A/70) created" + "; dpu2program4xi-created-by-retain2program4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            dpu2program4xi = create2programwithsource4dpu()
            message2print = "{(Step 4B/70) created" + "; dpu2program4xi-created-by-retain2program4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            dpu2program4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; dpu2program4xi-yielded-by-retain2program4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        dpu2program4xi['COUNT2REFERENCE'] = dpu2program4xi['COUNT2REFERENCE'] + task4retain2dpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2program4dpu: " + str(dpu2program4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1            
        if dpu2program4xi['COUNT2REFERENCE'] <= 0:
            dpu2program4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2program4dpu: " + " ** " + str(dpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            dpu2program4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2program4dpu: " + " ** " + str(dpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; dpu2program4xi-yielded-by-retain2program4dpu: " + str(dpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack     
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-retain2program4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return dpu2program4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2programinfo4dpu" usage = "
#
#   print('(Step 3B1/700) OF DPU2PROGRAM4XI5TEST - TRYING TO GET SOME PROGRAM BINARY INFO FOR DPU ', time4xi())     
#   task4getinfo2program = {}
#   task4getinfo2program ['FUNCTION2CALLBACK'] = 'Yes'
#   task4getinfo2program ['COUNT4BINARY'] = 'Yes'
#   task4getinfo2program ['DIRECTORY4BINARY'] = 'Yes'
#   task4getinfo2program ['CODE4BINARY'] = 'Yes'
#   task4getinfo2program ['SIZE4BINARY'] = 'Yes'
#   task4getinfo2program ['LANGUAGE4BINARY'] = 'Yes'
#   task4getinfo2program ['COUNT2REFERENCE'] = 'Yes'
#   task4getinfo2program ['CODE4BINARY'] = 'Yes'
#   try:
#       response4programinfo2dpu = get2programinfo4dpu(task4getinfo2program) 
#       print('(Step 3B2/700) OF DPU2PROGRAM4XI5TEST - GOT SOME PROGRAM BINARY INFO FOR DPU ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO GET SOME PROGRAM BINARY INFO FOR DPU', time4xi())                  
#   try:                                                                                          
#       keys2response = response4programinfo2dpu.keys()
#       print('COUNT2REFERENCE:', response4programinfo2dpu ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4programinfo2dpu ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4programinfo2dpu ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4programinfo2dpu ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4programinfo2dpu ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4programinfo2dpu ['SIZE4BINARY'])                
#       # 
#       keys2source = response4programinfo2dpu['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4programinfo2dpu ['CODE4BINARY'] [key2source] [1])
#           # COUNT2REFERENCE: 1
#           # COUNT4BINARY: 1
#           # DIRECTORY4BINARY: ...//__pycache__//
#           # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000004144AC8>
#           # LANGUAGE4BINARY: Python
#           # SIZE4BINARY: {'...//__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
#           # BINARY:  ...//__pycache__//dpu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'                       
#   except:
#       print('(Step 3Z4/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4programinfo2dpu : ', response4programinfo2dpu)
#   
#">

# -------------------------------------------------------------------------------------------------   
def get2programinfo4dpu(task4getinfo2program=None):

    '''
    gets dpu2program4xi and dpu2program4xi
    '''
    
    global dpu2program4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2programinfo4dpu"
    method2print = "get2programinfo4dpu"

    message2print = "{(Step 0A/33) started" + "; get2programinfo4dpu-started: " + str(get2programinfo4dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2programinfo4dpu at 0x0000000003A98BC8>
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2program-passed-to-get2programinfo4dpu: " + str(task4getinfo2program) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    dpu2program = {}
    try: 
        if dpu2program4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR DPU           
            # -------------------------------------------------------------------------------------           
            dpu2program = create2programwithsource4dpu()
            message2print = "{(Step 1B2/33) initialized" + "; dpu2program-initialized-by-get2programinfo4dpu to: " + str(dpu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; dpu2program4xi-using-by-get2programinfo4dpu: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF DPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2program is None:
            task4getinfo2program = {}
            dpu2program = dpu2program4xi
            message2print = "{(Step 1C/33) using" + "; dpu2program-got-by-get2programinfo4dpu: " + str(dpu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4programinfo = task4getinfo2program.keys()
        keys2task = task4getinfo2program.keys()
            
        # -------------------------------------------------------------------------------------
        # SOURCE OR BINARY PROGRAM INFO OF DPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2program4dpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                dpu2program [key2task] = dpu2program4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-dpu2program-set-by-get2programinfo4dpu to: " + str(dpu2program [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2program4dpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2program-using-by-get2program4dpu: " + str(task4getinfo2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; dpu2program4xi-using-by-get2program4dpu: " + str(dpu2program4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE PROGRAM WITH SOURCE DATA           
            # -------------------------------------------------------------------------------------           
            dpu2program = create2programwithsource4dpu()
            
            message2print = "{(Step 1F/33) initialized" + "; dpu2program-initialized-by-get2programinfo4dpu to: " + str(dpu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2program is None:
                dpu2program = dpu2program4xi
                message2print = "{(Step 1G/33) set" + "; dpu2program-set-by-get2programinfo4dpu to: " + str(dpu2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2program.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2program4dpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        dpu2program [key2task] = dpu2program4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-dpu2program -set-by-get2programinfo4dpu to: " + str(dpu2program  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2program4dpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            dpu2program = {}
            message2print = "{(Step 1Z3/70) yielded" + "; dpu2program-yielded-by-get2program4dpu: " + str(dpu2program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
     
    keys2program4dpu = dpu2program4xi.keys()
    for key2program4dpu in keys2program4dpu:
        message2print = "{(Step 70/70) returning" + "; key2program4dpu-returning-by-get2programinfo4dpu: " + str(key2program4dpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...
        
    return dpu2program
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2buildinfo4dpu" usage = "
#
#   response4get2buildinfo = get2buildinfo4dpu()
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2buildinfo4dpu(task4getinfo2build=None):

    '''
    gets dpu2build4xi and dpu2build4xi
    '''
    
    global dpu2build4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2buildinfo4dpu"
    method2print = "get2buildinfo4dpu"

    message2print = "{(Step 0A/33) started" + "; get2buildinfo4dpu-started: " + str(get2buildinfo4dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2build-passed-to-get2buildinfo4dpu: " + str(task4getinfo2build) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    dpu2build = {}
    try: 
        if dpu2build4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR DPU           
            # -------------------------------------------------------------------------------------           
            dpu2build = {}
            message2print = "{(Step 1B2/33) initialized" + "; dpu2build-initialized-by-get2buildinfo4dpu to: " + str(dpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; dpu2build4xi-using-by-get2buildinfo4dpu: " + str(dpu2build4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF DPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2build is None:
            task4getinfo2build = {}
            dpu2build = dpu2build4xi
            message2print = "{(Step 1C/33) using" + "; dpu2build-got-by-get2buildinfo4dpu: " + str(dpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4buildinfo = task4getinfo2build.keys()
        keys2task = task4getinfo2build.keys()
            
        # -------------------------------------------------------------------------------------
        # BUILD INFO OF DPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2build4dpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                dpu2build [key2task] = dpu2build4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-dpu2build-set-by-get2buildinfo4dpu to: " + str(dpu2build [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2build4dpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2build-using-by-get2build4dpu: " + str(task4getinfo2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; dpu2build4xi-using-by-get2build4dpu: " + str(dpu2build4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE BUILD           
            # -------------------------------------------------------------------------------------           
            dpu2build = {}
            message2print = "{(Step 1F/33) initialized" + "; dpu2build-initialized-by-get2buildinfo4dpu to: " + str(dpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2build is None:
                dpu2build = dpu2build4xi
                message2print = "{(Step 1G/33) set" + "; dpu2build-set-by-get2buildinfo4dpu to: " + str(dpu2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2build.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2build4dpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        dpu2build [key2task] = dpu2build4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-dpu2build -set-by-get2buildinfo4dpu to: " + str(dpu2build  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2build4dpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            dpu2build = {}
            message2print = "{(Step 1Z3/70) yielded" + "; dpu2build-yielded-by-get2build4dpu: " + str(dpu2build) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; dpu2build-returning-by-get2buildinfo4dpu: " + str(dpu2build) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 
    
    return dpu2build
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "link2program4dpu" usage = "
#     
#   print('(Step 2AB2/700) OF DPU2PROGRAM4XI5TEST - TRYING TO LINK INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4link = link2program4dpu()
#       print('(Step 2BB2/700) OF DPU2PROGRAM4XI5TEST - LINKED INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi()) 
#   except:
#       print('(Step 2Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO LINK INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                     
#   try:
#       print('(Step CC2/700) OF DPU2PROGRAM4XI5TEST - response4link:') 
#       keys2response = response4link.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4link[key2response])
#           # dpu2sampler4xi5test.py  :  <Process4Xi(Process4Xi-1, stopped[1])>
#   except:
#       print('(Step 2Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4link *** ', response4link)
#   tailor2trace4program(t2xi)                                                                              # TAILOR TRACE AGAIN - TARGET FUNCTIONS OF ACTIVATED PROCESSES MIGHT HAVE RESET TRACE ! 
#          
#">

# -------------------------------------------------------------------------------------------------   
def link2program4dpu(task4program2dpu=None):

    '''
    links program for DPU
    '''
    global dpu2program4xi                                                                                           # Program of DPU    
    global import2use4dpu                                                                                           # Imports of DPU
    global target2function4dpu                                                                                      # Target function of Process4Xi
       
                
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "link2program4dpu" 
    method2print = "link2program4dpu"

    message2print = "{(Step 1A/70) started" + "; link2program4dpu-started: " + str(link2program4dpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function link2program4dpu at 0x0000000004332448>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2dpu-to-link2program4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    process2program4dpu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4program2dpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2dpu-yielded-by-link2program4dpu: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - LINK SOURCES INSIDE PROGRAM OBJECT FOR DPU ! 
    # ---------------------------------------------------------------------------------------------    
    process2program4dpu = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; process2program4dpu-initialized-by-link2program4dpu to: " + str(process2program4dpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                  
        # ---------------------------------------------------------------------------------------------             
        keys2source4dpu = dpu2program4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2source4dpu-using-by-link2program4dpu: " + str(keys2source4dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4dpu + 'dpu2sampler4xi5test.py'])
        
        unit4processing = "dpu"
        
        for key2source4dpu in keys2source4dpu:
            index2search=str.find(key2source4dpu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2link = key2source4dpu[index2search:]                                                          # dpu2sampler4xi5test.py
                index2search=str.find(target2link,".py")
                if index2search == -1:
                    pass
                else:
                    target2link4dpu = target2link[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2link4dpu-set-by-link2program4dpu to: " + str(target2link4dpu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # dpu2sampler4xi5test

                    p2source4dpu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4DPU 
                        # ---------------------------------------------------------------------------------------------    
      
                        try:
                            import2use4dpu.get(target2link4dpu, handle2error4dpu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2link4dpu-imported-by-link2program4dpu: " + str(target2link4dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # dpu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4dpu-failed-by-link2program4dpu: " + str(import2use4dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2link4dpu-used-by-link2program4dpu: " + str(target2link4dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4DPU
                        # ---------------------------------------------------------------------------------------------                          
                        p2dpu = Process4Xi(target=target2function4dpu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2dpu-created-by-link2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-2, initial)>
                        
                        process2program4dpu[key2source4dpu] = p2dpu
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2dpu.start()
                            message2print = "{(Step 34A/70) started" + "; p2dpu-started-by-link2program4dpu: " + str(p2dpu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-2
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4dpu = p2dpu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-link2program4dpu: " + str(status2process4dpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2dpu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2dpu-run-by-link2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-link2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-link2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-link2program4dpu: " + str(p2dpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2source4dpu-failed-by-link2program4dpu: " + str(p2source4dpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    

 
        message2print = "{(Step 38A/70) set" + "; process2program4dpu-set-by-link2program4dpu to: " + str(process2program4dpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; source-yielded-by-link2program4dpu: " + str(dpu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    dpu2program4xi ['ACK2RETURN']  = code2ack
    dpu2program4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; process2program4dpu-returning-by-link2program4dpu: " + str(process2program4dpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    return process2program4dpu
            
#</function4xi> 
 
                                        
# =================================================================================================
#<class4xi class = "DPU2Program4Xi" base = "object" child = "None"
#   synopsis = "offers  methods for DPU programs" usage = "Program4Xi to DPU"
#>
# =================================================================================================
class DPU2Program4Xi(object): 
    ''' 
    DPU2Program4Xi offers  methods for DPU programs
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2dpu = cpu_corecount4xi()
    #"
    #/>
       
       
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (

            'add2programwithbinary',           # ACK 5.6.1   p132            
            'add2programwithsource',           # ACK 5.6.1   p133  
                                
            'build2program',                   # ACK 5.6.2   p137      

            'compile2program',                 # ACK 5.6.3   p139            
            'create2programwithsource',        # ACK 5.6.1   p132 
            'create2programwithbinary',        # ACK 5.6.1   p133 
            'create2programwithbuiltinkernels',# NACK 5.6.1  p135   
                       
            'delete2programwithbinary',        # ACK  
            'delete2programwithsource',        # ACK               
            
            'fetch2programbinary',             # ACK
            'fetch2programsource',             # ACK
            
            'get2programinfo',                 # ACK 5.6.7   p150 
            'get2programbuildinfo',            # NACK 5.6.7  p153
            
            'link2program',                    # ACK 5.6.3   p142                          
            'release2program',                 # ACK 5.6.1   p136                                         
            'retain2program',                  # ACK 5.6.1   p136
 
            'set2programwithsource',           # ACK
            'set2programwithbinary',           # ACK            
            'set2programcallback',             # ACK 
                              
            'trace2program',                   # ACK
            'unload2compiler',                 # NACK 5.6.6  p149 
        
            'sense',                           # ACK
            'reason',                          # ACK
            'ace')                             # ACK   
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-DPU2Program4Xi" usage = "
    # 
    #    print('(Step 110A/700) OF DPU2PROGRAM4XI5TEST - CREATING PROGRAM4DPU OBJECT ', time4xi())        
    #    try:
    #        # ---------------------------------------------------------------------------------------------
    #        # CREATE WITH OBJECT                  
    #        # ---------------------------------------------------------------------------------------------        
    #        programobject2dpu = DPU2Program4Xi()
    #        print('(Step 110B/700) OF DPU2PROGRAM4XI5TEST - CREATED PROGRAM4DPU OBJECT: ', programobject2dpu)                                  
    #        # <dpu2program4xi.DPU2Program4Xi object at 0x0000000003F69198>
    #            
    #        except:
    #    print('(Step 110Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM4DPU OBJECT')
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4program2dpu = None):
        '''
        _init__ method initializes DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "DPU2Program4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-DPU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 

        message2print = "{(Step 1B/33) borning" + "; dpu2program2xi-of-DPU2Program4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-dpu2program2xi-of-DPU2Program4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2dpu-sharing-by-dpu2program2xi-of-DPU2Program4Xi: " + str(DPU2Program4Xi.classvariable2dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #         
                                
        message2print = "{(Step 1E/33) passed" + "; task4program2dpu-passed-to-init-of-DPU2Program4Xi: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #                       
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33) - source!              
        # -----------------------------------------------------------------------------------------
    
        message2print = "{(Step 2A/33) passing" + "; task4program2dpu-passing-to-create2programwithsource4dpu-by-init-of-DPU2Program4Xi: " + str(task4program2dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
                
        dpu2program2xi = create2programwithsource4dpu(task4program2dpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2programwithsource4dpu-performed-by-init-of-DPU2Program4Xi: " + str(create2programwithsource4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2programwithsource4dpu at 0x0000000003A98D48>
       
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; dpu2program2xi-of-DPU2Program4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # dpu2program2xi-of-DPU2Program4Xi-activated
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-dpu2program2xi-of-DPU2Program4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2dpu-shared-by-dpu2program2xi-of-DPU2Program4Xi: " + str(DPU2Program4Xi.classvariable2dpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4         
       
        message2print = "{(Step 3D/33) returning" + "; init-of-DPU2Program4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2programwithbinary-of-DPU2Program4Xi" usage = "
    #  
    #   print('(Step 132A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO ADD BINARY INTO PROGRAM OBJECT WITH METHOD ', time4xi())         
    #   try:
    #       task4add2program = {}
    #       task4add2program ['CODE4BINARY'] = {}
    #       task4add2program ['SIZE4BINARY'] = {}            
    #       # NOT OK - EXISTS
    #       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] = {}
    #       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
    #       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
    #       task4add2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc'])
    #                
    #       # OK
    #       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = {}
    #       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
    #       task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
    #       task4add2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'])
    #       
    #       response4add2programwithbinary = programobject2dpu.add2programwithbinary(task4add2program) 
    #       print('(Step 132B/700) OF DPU2PROGRAM4XI5TEST - ADDED BINARY INTO PROGRAM OBJECT WITH METHOD ', time4xi())
    #   except:
    #       print('(Step 132Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO ADD BINARY INTO PROGRAM OBJECT WITH METHOD *** ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4add2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4add2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4add2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4add2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4add2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4add2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4add2programwithbinary ['SIZE4BINARY'])                
    #       # 
    #       keys2source = response4add2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4add2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 2
    #       # DIRECTORY4BINARY: ... //__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x000000000419BBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'...//__pycache__//dpu2image4xi.cpython-33.pyc': 1006, '...//__pycache__//dpu3image4xi.cpython-33.pyc': 2}
    #       # BINARY:  ...__pycache__//dpu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #       # BINARY:  ...//__pycache__//dpu3image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'       
    #   except:
    #       print('(Step 132Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4ADD2PROGRAMWITHBINARY : ', response4add2programwithbinary) 
    #  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def add2programwithbinary(self, task4add2programwithbinary = None):
        '''
        sets program DPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "add2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; add2programwithbinary-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:02 11/22/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4add2programwithbinary-passed-to-add2programwithbinary-of-DPU2Program4Xi: " + str(task4add2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4BINARY': {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
        #                  folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
        #  'SIZE4BINARY': {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': 2, 
        #                  folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': 2}
        # }

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2programwithbinary-passing-to-add2programwithbinary4dpu-by-add2programwithbinary-of-DPU2Program4Xi: " + str(task4add2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4BINARY': {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
        #                  folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
        #  'SIZE4BINARY': {folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': 2, 
        #                  folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc': 2}
        
        response4add2programwithbinary = add2programwithbinary4dpu(task4add2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; add2programwithbinary4dpu-performed-by-add2programwithbinary-of-DPU2Program4Xi: " + str(add2programwithbinary4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function add2programwithbinary4dpu at 0x00000000040EDCC8>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2programwithbinary-returning-by-add2programwithbinary-of-DPU2Program4Xi: " + str(response4add2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 1 ...
        
                
        return response4add2programwithbinary
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2programwithsource-of-DPU2Program4Xi" usage = "
    #  
    #   print('(Step 122A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO ADD SOURCE INTO PROGRAM OBJECT WITH CLASS ', time4xi())          
    #   try:
    #       task4add2program = {}
    #       # 
    #       task4add2program ['CODE4SOURCE'] = {}
    #       task4add2program ['SIZE4SOURCE'] = {}
    #                   
    #       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = {}
    #       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] [1] = '... profile = "33" />\n'
    #       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] [2] = '... program = "dpu2buffer4xi.py" />\n'
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'])
    #       # 
    #       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] = {}
    #       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] [1] = '... profile = "33" />\n'
    #       task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] [2] = '... program = "dpu2sampler4xi5test.py" />\n'
    #       
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2sampler4xi5test.py'])            
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'])            
    #       
    #       response4add2programwithsource = programobject2dpu.add2programwithsource(task4add2program)
    #       print('(Step 122B/700) OF DPU2PROGRAM4XI5TEST - ADDED SOURCE INTO PROGRAM OBJECT WITH CLASS ', time4xi())    
    #   except:
    #       print('(Step 122Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO ADD SOURCE INTO PROGRAM OBJECT WITH CLASS *** ', time4xi())
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4add2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4add2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4add2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4add2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4add2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4add2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT4SOURCE:  2
    #       # DIRECTORY4SOURCE:  OK
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {' ... //dpu2buffer4xi.py': 2, '... dpu2sampler4xi5test.py': 1337}
    #       # SOURCE:  ... //dpu2buffer4xi.py
    #       # ... profile = "33" />
    #       # 
    #       # SOURCE:  ... dpu2sampler4xi5test.py
    #       # ... profile = "11" />                              
    #   except:
    #       print('(Step 1222Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4ADD2PROGRAMWITHSOURCE: ', response4add2programwithsource)  
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def add2programwithsource(self, task4add2programwithsource = None):
        '''
        sets program DPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "add2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; add2programwithsource-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:01 11/22/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4add2programwithsource-passed-to-add2programwithsource-of-DPU2Program4Xi: " + str(task4add2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "dpu2buffer4xi.py" />\n'}, 
        #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "dpu2sampler4xi5test.py" />\n'}}, 
        #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 2, 
        #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': 2}
        #}

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2programwithsource-passing-to-add2programwithsource4dpu-by-add2programwithsource-of-DPU2Program4Xi: " + str(task4add2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "dpu2buffer4xi.py" />\n'}, 
        #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "dpu2sampler4xi5test.py" />\n'}}, 
        #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 2, 
        #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': 2}
        #}; time: 00:44:01 11/22/11 Pacific Standard Time}
        
        response4add2programwithsource = add2programwithsource4dpu(task4add2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; add2programwithsource4dpu-performed-by-add2programwithsource-of-DPU2Program4Xi: " + str(add2programwithsource4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function add2programwithsource4dpu at 0x00000000040DDD48>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2programwithsource-returning-by-add2programwithsource-of-DPU2Program4Xi: " + str(response4add2programwithsource) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'Python', 
        #  'COUNT4SOURCE': 2, 
        #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 2, 
        #                  folder2lib4dpu + 'dpu2sampler4xi5test.py': 1337}, 
        #  'DATA2CALLBACK': ...
         
        return response4add2programwithsource
                                
                            
    #</methodize4xi>    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "build2program-of-DPU2Program4Xi" usage = "
    # 
    #   task4build2program  = {}
    #   task4build2program  ['PROGRAM'] = folder2lib4dpu + 'dpu2sampler4xi.py'
    #   task4build2program  ['OPTIONS'] = {}
    #   task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
    #   task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
    #   task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
    #   task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'
    #   print('(Step 240A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO BUILD PROGRAM OF DPU DEVICE ', time4xi())    
    #   try:                               
    #       response4build2program = programobject2dpu.build2program(task4build2program) 
    #       print('(Step 240B/700) OF DPU2PROGRAM4XI5TEST - BUILT PROGRAM OF DPU DEVICE ', time4xi())
    #   except:
    #       print('(Step 240Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO BUILD PROGRAM OF DPU DEVICE ', time4xi())
    #   try:                                                                                                     
    #       keys2response = response4build2program.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4build2program[key2response])
    #   # ACK2RETURN  :   11A1-12A1-
    #   # NACK2RETURN  :   13Z1-14Z1-
    #   # OPTIONS  :  {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
    #   # PROGRAM  :  folder2lib4dpu + dpu2sampler4xi.py                
    #   except:
    #       print('(Step 240Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT ', response4build2program)              
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def build2program(self, task4build = None):
        '''
        build2program method builds program objects of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "build2program"
                     
        message2print = "{(Step 1A/33) started" + "; build2program-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4build-passed-to-create-of-DPU2Program4Xi: " + str(task4build) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4build-passing-to-build2program4dpu-by-build2program-of-DPU2Program4Xi: " + str(task4build) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #    
  
        dpu2program2xi = build2program4dpu(task4build)

        message2print = "{(Step 2B/33) performed" + "; build2program4dpu-performed-by-build2program-of-DPU2Program4Xi: " + str(build2program4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; dpu2program2xi-returning-by-build2program-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return dpu2program2xi
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "compile2program-of-DPU2Program4Xi" usage = "
    # 
    #   print('(Step 121A1/700) OF DPU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR DPU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # COMPILE                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4compile = programobject2dpu.compile2program()
    #       print('(Step 121A2/700) OF DPU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR DPU', time4xi())                                   
    #   except:
    #       print('(Step 121AZ1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO COMPILE PROGRAM FOR DPU')
    #   try:
    #       keys2response4compile = response4compile.keys()
    #       for key2response4compile in keys2response4compile:
    #           print('(Step 121A3/700) OF DPU2PROGRAM4XI5TEST - KEY OF RESPONSE4COMPILE: ', key2response4compile)
    #           # dpu2sampler4xi5test.py 
    #   except:
    #       print('(Step 121AZ2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4COMPILE:' , response4compile)
    #                
    #   tailor2trace4program(t2xi)                  # <-- tailor trace again - target functions of activated processes might have reset trace !
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def compile2program(self, task4program2create = None):
        '''
        compile2program compiles for program object of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "compile2program"
                     
        message2print = "{(Step 1A/33) started" + "; compile2program-of-DPU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 03:09:40 11/23/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-compile2program4dpu-by-compile2program-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None
  
        dpu2program2xi = compile2program4dpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; compile2program4dpu-performed-by-compile2program-of-DPU2Program4Xi: " + str(compile2program4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; dpu2program2xi-returning-by-compile2program-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return dpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "link2program-of-DPU2Program4Xi" usage = "
    # 
    #   print('(Step 121C1/700) OF DPU2PROGRAM4XI5TEST - LINKING PROGRAM FOR DPU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # LINK                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4link = programobject2dpu.link2program()
    #       print('(Step 121C2/700) OF DPU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR DPU', time4xi())                                   
    #   except:
    #       print('(Step 121CZ1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO LINK PROGRAM FOR DPU')
    #   try:
    #       keys2response4link = response4link.keys()
    #       for key2response4link in keys2response4link:
    #           print('(Step 121C3/700) OF DPU2PROGRAM4XI5TEST - KEY OF RESPONSE4LINK: ', key2response4link)
    #           # 
    #   except:
    #       print('(Step 121CZ2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4LINK:' , response4link)
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def link2program(self, task4program2create = None):
        '''
        link2program links for program object of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "link2program"
                     
        message2print = "{(Step 1A/33) started" + "; link2program-of-DPU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 04:02:10 11/23/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-link2program4dpu-by-link2program-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None
  
        dpu2program2xi = link2program4dpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; link2program4dpu-performed-by-link2program-of-DPU2Program4Xi: " + str(link2program4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; dpu2program2xi-returning-by-link2program-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return dpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2programwithsource-of-DPU2Program4Xi" usage = "
    # 
    #   print('(Step 190A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM WITH SOURCE FOR DPU ', time4xi()) 
    #   try:
    #       programobject2dpu = DPU2Program4Xi()                            
    #       response4create2programwithsource = programobject2dpu.create2programwithsource()
    #       print('(Step 190B/700) OF DPU2PROGRAM4XI5TEST - CREATED PROGRAM WITH SOURCE FOR DPU ', time4xi())
    #   except:
    #       print('(Step 190Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM WITH SOURCE FOR DPU ', time4xi())                                    
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4create2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4create2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4create2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4create2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4create2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [3])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [4])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [5])
    #           
    #       # COUNT4SOURCE:  1
    #       # DIRECTORY4SOURCE:  folder2lib4dpu
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {folder2lib4dpu + 'dpu2sampler4xi5test.py': 1337}
    #       # SOURCE:  folder2lib4dpu + dpu2sampler4xi5test.py
    #       # ... profile = "11" />
    #       # ... program = "dpu2sampler4xi5test.py">
    #       # ... abstract = "tests dpu2sampler4xi.py"/>
    #       # ... verification = "11" target = "dpu2sampler4xi.py"/>              
    #   except:
    #       print('(Step 190Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithsource *** ', response4create2programwithsource)  
    #           
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2programwithsource(self, task4program2create = None):
        '''
        create2programwithsource method creates program objects of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "create2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; create2programwithsource-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:16:42 11/17/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-create2programwithsource4dpu-by-create2programwithsource-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None   
  
        dpu2program2xi = create2programwithsource4dpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; create2programwithsource4dpu-performed-by-create2programwithsource-of-DPU2Program4Xi: " + str(create2programwithsource4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2programwithsource4dpu at 0x0000000004024848> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; dpu2program2xi-returning-by-create2programwithsource-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'Python', 
        #  'COUNT4SOURCE': 1, 
        #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2sampler4xi5test.py': 1337}, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4dpu-failed ; time: 05:16:42 11/17/11 Pacific Standard Time}', 
        # ...

        return dpu2program2xi
                                
    #                        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2programwithbinary-of-DPU2Program4Xi" usage = "
    # 
    #   print('(Step 130A/700) OF DPU2PROGRAM4XI5TEST - CREATING WITH BINARY PROGRAM FOR DPU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # CREATE BINARY                         
    #       # ---------------------------------------------------------------------------------------------        
    #       response4create2programwithbinary = programobject2dpu.create2programwithbinary()
    #       print('(Step 130A/700) OF DPU2PROGRAM4XI5TEST - CREATED WITH BINARY PROGRAM FOR DPU', time4xi())
    #   except:
    #       print('(Step 130Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO CREATE  WITH BINARY PROGRAM FOR DPU')  
    #   try:                                                                                          
    #       keys2response = response4create2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4create2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4create2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4create2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4create2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4create2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4create2programwithbinary ['SIZE4BINARY'])                
    #       #
    #       keys2source = response4create2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4create2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 1
    #       # DIRECTORY4BINARY: folder2lib4dpu + __pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x000000000415CBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'folder2lib4dpu + '__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
    #       # BINARY:  folder2lib4dpu + __pycache__//dpu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #   except:
    #       print('(Step 130Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4CREATE2PROGRAMWITHBINARY : ', response4create2programwithbinary)              
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2programwithbinary(self, task4program2create = None):
        '''
        create2programwithbinary method creates program objects of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "create2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; create2programwithbinary-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:16:42 11/17/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-create2programwithbinary4dpu-by-create2programwithbinary-of-DPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None   
  
        dpu2program2xi = create2programwithbinary4dpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; create2programwithbinary4dpu-performed-by-create2programwithbinary-of-DPU2Program4Xi: " + str(create2programwithbinary4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2programwithbinary4dpu at 0x00000000040248C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; dpu2program2xi-returning-by-create2programwithbinary-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0, 
        #  'COUNT2REFERENCE': 1, 
        #  'DIRECTORY4BINARY': folder2lib4dpu + '__pycache__//', 
        # ... 

        return dpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2programwithbinary-of-DPU2Program4Xi" usage = "
    #  
    #   print('(Step 134A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO DELETE BINARY INSIDE PROGRAM OBJECT WITH METHOD ', time4xi())        
    #   try:
    #       task4delete2program = {}
    #       task4delete2program ['CODE4BINARY'] = {}
    #       task4delete2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = 'Yes'
    #       task4delete2program ['CODE4BINARY'] [folder2lib4dpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
    #                task4delete2program ['SIZE4BINARY'] = {}
    #       task4delete2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'] = 'Yes'
    #       task4delete2program ['SIZE4BINARY'] [folder2lib4dpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
    #       
    #       response4delete2programwithbinary = programobject2dpu.delete2programwithbinary(task4delete2program)
    #       print('(Step 134B/700) OF DPU2PROGRAM4XI5TEST - DELETED BINARY INSIDE PROGRAM OBJECT WITH METHOD ', time4xi())                                                                  
    #   except:
    #       print('(Step 134Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO DELETE BINARY INSIDE PROGRAM OBJECT WITH METHOD *** ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4delete2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4delete2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4delete2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4delete2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4delete2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4delete2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4delete2programwithbinary ['SIZE4BINARY'])                
    #       # 
    #       keys2source = response4delete2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4delete2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 1
    #       # DIRECTORY4BINARY: ...//__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x000000000412BBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'...//__pycache__//dpu2image4xi.cpython-33.pyc': 1006}
    #       # BINARY:  ...__pycache__//dpu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #                       
    #   except:
    #       print('(Step 134Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithbinary : ', response4delete2programwithbinary)         
    #                  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def delete2programwithbinary(self, task4delete2programwithbinary = None):
        '''
        sets program DPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "delete2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; delete2programwithbinary-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2programwithbinary-passed-to-delete2programwithbinary-of-DPU2Program4Xi: " + str(task4delete2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2programwithbinary-passing-to-delete2programwithbinary4dpu-by-delete2programwithbinary-of-DPU2Program4Xi: " + str(task4delete2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4delete2programwithbinary = delete2programwithbinary4dpu(task4delete2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; delete2programwithbinary4dpu-performed-by-delete2programwithbinary-of-DPU2Program4Xi: " + str(delete2programwithbinary4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2programwithbinary-returning-by-delete2programwithbinary-of-DPU2Program4Xi: " + str(response4delete2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4delete2programwithbinary
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2programwithsource-of-DPU2Program4Xi" usage = "
    #  
    #   print('(Step 124A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO DELETE SOURCE IN PROGRAM OBJECT WITH CLASS ', time4xi())        
    #   try:
    #       task4delete2program = {}
    #       task4delete2program ['CODE4SOURCE'] = {}
    #       task4delete2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = 'Yes'
    #       task4delete2program ['CODE4SOURCE'] [folder2lib4dpu + 'dpu2dummy4xi.py'] = 'Yes'
    #                task4delete2program ['SIZE4SOURCE'] = {}
    #       task4delete2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2buffer4xi.py'] = 'Yes'
    #       task4delete2program ['SIZE4SOURCE'] [folder2lib4dpu + 'dpu2dummy4xi.py'] = 'Yes'
    #       
    #       response4delete2programwithsource = programobject2dpu.delete2programwithsource(task4delete2program)
    #       print('(Step 124B/700) OF DPU2PROGRAM4XI5TEST - DELETED SOURCE IN PROGRAM OBJECT WITH CLASS ', time4xi())
    #   except:
    #       print('(Step 124Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO DELETE SOURCE IN PROGRAM OBJECT WITH CLASS *** ', time4xi())    
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4delete2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4delete2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4delete2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4delete2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4delete2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4delete2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT4SOURCE:  1
    #       # DIRECTORY4SOURCE:  OK 
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {'... dpu2sampler4xi5test.py': 1337}
    #       # SOURCE:  ... dpu2sampler4xi5test.py
    #       # ... profile = "11" />                        
    #   except:
    #       print('(Step 124Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithsource *** ', response4delete2programwithsource)                            
    #                    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def delete2programwithsource(self, task4delete2programwithsource = None):
        '''
        sets program DPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "delete2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; delete2programwithsource-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:01 11/22/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2programwithsource-passed-to-delete2programwithsource-of-DPU2Program4Xi: " + str(task4delete2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 'Yes', 
        #                  folder2lib4dpu + 'dpu2dummy4xi.py': 'Yes'}, 
        #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 'Yes', 
        #  folder2lib4dpu + 'dpu2dummy4xi.py': 'Yes'}}

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2programwithsource-passing-to-delete2programwithsource4dpu-by-delete2programwithsource-of-DPU2Program4Xi: " + str(task4delete2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 'Yes', 
        #                  folder2lib4dpu + 'dpu2dummy4xi.py': 'Yes'}, 
        #  'SIZE4SOURCE': {folder2lib4dpu + 'dpu2buffer4xi.py': 'Yes', 
        #  folder2lib4dpu + 'dpu2dummy4xi.py': 'Yes'}}
        
        response4delete2programwithsource = delete2programwithsource4dpu(task4delete2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; delete2programwithsource4dpu-performed-by-delete2programwithsource-of-DPU2Program4Xi: " + str(delete2programwithsource4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function delete2programwithsource4dpu at 0x00000000040DDE48>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2programwithsource-returning-by-delete2programwithsource-of-DPU2Program4Xi: " + str(response4delete2programwithsource) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'LANGUAGE4SOURCE': 'Python' ...
        
                
        return response4delete2programwithsource
                                
                            
    #</methodize4xi>    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programwithsource-of-DPU2Program4Xi" usage = "
    # 
    #   ... prepare task4set2program ...
    #   try: 
    #       response4set2program = programobject2dpu.set2programwithsource(task4set2program)
    #       print('(Step 158B/700) OF DPU2PROGRAM4XI5TEST - SET PROGRAM SOURCE FOR DPU: ', time4xi())    
    #   except:
    #       print('(Step 158Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM SOURCE FOR DPU: ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4set2program.keys()
    #       print('COUNT2REFERENCE:', response4set2program ['COUNT2REFERENCE'])   
    #       print('COUNT4SOURCE:', response4set2program ['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE:', response4set2program ['DIRECTORY4SOURCE']) 
    #       print('FUNCTION2CALLBACK:', response4set2program ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4SOURCE:', response4set2program ['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE:', response4set2program ['SIZE4SOURCE'])                
    #            keys2source = response4set2program['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4set2program ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4SOURCE: 1
    #       # DIRECTORY4SOURCE: OK
    #       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000003AC5BC8>
    #       # LANGUAGE4SOURCE: Python
    #       # SIZE4SOURCE: 1
    #       # SOURCE:  ... //dpu2image4xi5test.py
    #       # ... profile = "11" />        
    #   except:
    #       print('(Step 158Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4set2program : ', response4set2program)  
    #       
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2programwithsource(self, task4set2programwithsource = None):
        '''
        sets program DPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "set2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; set2programwithsource-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programwithsource-passed-to-set2programwithsource-of-DPU2Program4Xi: " + str(task4set2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programwithsource-passing-to-set2programwithsource4dpu-by-set2programwithsource-of-DPU2Program4Xi: " + str(task4set2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        dpu2program2xi = set2programwithsource4dpu(task4set2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; set2programwithsource4dpu-performed-by-set2programwithsource-of-DPU2Program4Xi: " + str(set2programwithsource4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dpu2program2xi-returning-by-set2programwithsource-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return dpu2program2xi
                                
                            
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programwithbinary-of-DPU2Program4Xi" usage = "
    # 
    #   ... prepare task4set2programwithbinary ...
    #   task4set2programwithbinary ['CODE4BINARY'] = None
    #   print('(Step 167A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO SET BINARY OF TASK FOR PROGRAM4DPU  ', time4xi())      
    #   try: 
    #       binary4dpu = {}
    #       key2binary4dpu = folder2lib4dpu + '__pycache__//dpu3image4xi.cpython-33.pyc'
    #       code2binary4dpu = {}
    #       code2binary4dpu [1] = b'l\x0c\r\n'
    #       code2binary4dpu [2] = b'\x00Z\n'
    #       
    #       binary4dpu [key2binary4dpu] = code2binary4dpu
    #                                 
    #       task4set2programwithbinary ['CODE4BINARY'] = binary4dpu                                                                                                              
    #       print('(Step 167B/700) OF DPU2PROGRAM4XI5TEST - SET BINARY OF TASK FOR PROGRAM4DPU TO: ', task4set2programwithbinary ['CODE4BINARY'])                           
    #   except:
    #       print('(Step 167Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO SET BINARY OF TASK FOR PROGRAM4DPU  ', time4xi()) 
    #       
    #   print('(Step 167CB/700) OF DPU2PROGRAM4XI5TEST - TRYING TO SET BINARY PROGRAM FOR DPU: ', time4xi())      
    #   try: 
    #       response4set2program = programobject2dpu.set2programwithbinary(task4set2programwithbinary)
    #       print('(Step 167D/700) OF DPU2PROGRAM4XI5TEST - SET BINARY PROGRAM FOR DPU: ', time4xi())
    #   except:
    #       print('(Step 167Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4SET2PROGRAM: ', response4set2program)                     
    #   try:                                                                                          
    #       keys2response = response4set2program.keys()
    #       print('COUNT2REFERENCE:', response4set2program ['COUNT2REFERENCE']) 
    #   except:
    #       pass
    #   try:        
    #       print('COUNT4BINARY:', response4set2program ['COUNT4BINARY'])
    #   except:
    #       pass
    #   try:        
    #       print('DIRECTORY4BINARY:', response4set2program ['DIRECTORY4BINARY']) 
    #   except:
    #       pass
    #   try:        
    #       print('FUNCTION2CALLBACK:', response4set2program ['FUNCTION2CALLBACK'])   
    #   except:
    #       pass
    #   try:        
    #       print('LANGUAGE4BINARY:', response4set2program ['LANGUAGE4BINARY'])
    #   except:
    #       pass
    #   try:        
    #       print('SIZE4BINARY:', response4set2program ['SIZE4BINARY'])                
    #   except:
    #       pass
    #   try:        
    #            keys2binary = response4set2program['CODE4BINARY'].keys()
    #       for key2binary in keys2binary:
    #           print('BINARY: ', key2binary)
    #           print(response4set2program ['CODE4BINARY'] [key2binary] [1])
    #       #    
    #   except:
    #       pass
    #       # COUNT2REFERENCE: 1
    #       # DIRECTORY4BINARY: ... //__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4dpu at 0x0000000003B1FBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: 1
    #       # BINARY:  ... //__pycache__//dpu3image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'           
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2programwithbinary(self, task4set2programwithbinary = None):
        '''
        sets program DPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "set2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; set2programwithbinary-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:32:44 11/17/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programwithbinary-passed-to-set2programwithbinary-of-DPU2Program4Xi: " + str(task4set2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'C99', 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C68EF0>, 
        #  'CONTEXT': ...

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programwithbinary-passing-to-set2programwithbinary4dpu-by-set2programwithbinary-of-DPU2Program4Xi: " + str(task4set2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4set2programwithbinary = set2programwithbinary4dpu(task4set2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; set2programwithbinary4dpu-performed-by-set2programwithbinary-of-DPU2Program4Xi: " + str(set2programwithbinary4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4set2programwithbinary-returning-by-set2programwithbinary-of-DPU2Program4Xi: " + str(response4set2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4set2programwithbinary
                                
                            
    #</methodize4xi>    
    

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2programinfo-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF DPU2PROGRAM4XI5TEST - TRYING TO GET PROGRAM INFO ', time4xi())    
    #   try:                               
    #       response4get2programinfo = programobject2dpu.get2programinfo()                                 
    #                                
    #       keys2response = response4get2programinfo.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4get2programinfo[key2response])
    #           # CODE4SOURCE  :  {folder2lib4dpu + 'dpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ....
    #           
    #   except:
    #       print('(Step 106Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO GET PROGRAM INFO ', time4xi()) 
    #   
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2programinfo(self, task4getinfo2program=None):
        '''
        get2programinfo method gets objects of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "get2programinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2programinfo-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:01:22 11/17/11 Pacific Standard Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        dpu2program2xi = get2programinfo4dpu(task4getinfo2program)
        
        message2print = "{(Step 2A/33) returning" + "; dpu2program2xi-returning-by-get2programinfo-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'COUNT2REFERENCE': 1, 
        #  'DIRECTORY4BINARY': folder2lib4dpu + '__pycache__//', 
        #  'LANGUAGE4BINARY': 'Python', 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4dpu-failed ; time: 05:16:42 11/17/11 Pacific Standard Time}', 
        #  'CLASS': 'create2programwithbinary4dpu', 
        #  'METHOD': 'create2programwithbinary4dpu'}, 
        #  'CODE4BINARY': ...

        return dpu2program2xi
                            
    #</methodize4xi>

   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2buildinfo-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF DPU2BUILD4XI5TEST - TRYING TO GET BUILD INFO ', time4xi())    
    #   try:                               
    #                               
    #       response4get2buildinfo = programobject2dpu.get2buildinfo()
    #       print('(Step 106A2/700) OF DPU2BUILD4XI5TEST - GOT BUILD INFO ', time4xi())    
    #   except:
    #       print('(Step 106Z1/700) OF DPU2BUILD4XI5TEST *** FAILED TO GET BUILD INFO ', time4xi()) 
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2buildinfo(self, task4getinfo2build=None):
        '''
        get2buildinfo method gets objects of DPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "get2buildinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2buildinfo-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        dpu2build2xi = get2buildinfo4dpu(task4getinfo2build)
        
        message2print = "{(Step 2A/33) returning" + "; dpu2build2xi-returning-by-get2buildinfo-of-DPU2Program4Xi: " + str(dpu2build2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        return dpu2build2xi
                            
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2program-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 170A/700) OF DPU2PROGRAM4XI5TEST - RELEASING PROGRAM', time4xi())  
    #   task4release2program  = {}
    #   task4release2program  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4release2program = programobject2dpu.release2program(task4release2program)
    #       print('(Step 170B/700) OF DPU2PROGRAM4XI5TEST - RELEASED PROGRAM', time4xi())
    #   except:
    #       print('(Step 170Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO RELEASE PROGRAM', time4xi())                        
    #       keys2response = response4release2program.keys()
    #   try:
    #       print('(Step 170C/700) OF DPU2PROGRAM4XI5TEST - COUNT2REFERENCE: ', response4release2program['COUNT2REFERENCE'] )
    #       # 0
    #   except:
    #       print('(Step 170Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RELEASE2PROGRAM', response4release2program)  
    #  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2program(self, task4release2program = None):
        '''
        releases DPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "release2program"
                     
        message2print = "{(Step 1A/33) started" + "; release2program-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2program-passed-to-release2program-of-DPU2Program4Xi: " + str(task4release2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2program-passing-to-release2program4dpu-by-release2program-of-DPU2Program4Xi: " + str(task4release2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        dpu2program2xi = release2program4dpu(task4release2program)

        message2print = "{(Step 2B/33) performed" + "; release2program4dpu-performed-by-release2program-of-DPU2Program4Xi: " + str(release2program4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dpu2program2xi-returning-by-release2program-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #               
                
        return dpu2program2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2program-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 180A/700) OF DPU2PROGRAM4XI5TEST - RETAINING PROGRAM ', time4xi())
    #   task4retain2program  = {}
    #   task4retain2program  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4retain2program = programobject2dpu.retain2program(task4retain2program)
    #       print('(Step 180B/700) OF DPU2PROGRAM4XI5TEST - RETAINED PROGRAM ', time4xi())
    #   except:
    #       print('(Step 180Z1/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO RETAIN PROGRAM ', time4xi())                    
    #   try:
    #       print('(Step 180C/700) OF DPU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4retain2program ['COUNT2REFERENCE'])
    #       # 2
    #   except:
    #       print('(Step 180Z2/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RETAIN2PROGRAM ', response4retain2program)
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2program(self, task4retain2program = None):
        '''
        retains DPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "retain2program"
                     
        message2print = "{(Step 1A/33) started" + "; retain2program-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2program-of-DPU2Program4Xi: " + str(task4retain2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2program4dpu-by-retain-of-DPU2Program4Xi: " + str(task4retain2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
  
        dpu2program2xi = retain2program4dpu(task4retain2program)

        message2print = "{(Step 2B/33) performed" + "; retain2program4dpu-performed-by-retain2program-of-DPU2Program4Xi: " + str(retain2program4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; dpu2program2xi-returning-by-retain2program-of-DPU2Program4Xi: " + str(dpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
            
                
        return dpu2program2xi
                                                           
    #</methodize4xi>   
    
 
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2program-of-DPU2Program4Xi" usage = "
    #     
    #   print('(Step 210A/25) OF DPU2PROGRAM4XI5TEST - TRYING TO TRACE DPU: ', time4xi())      
    #   trace2dpu = None
    #   try:
    #       trace2dpu = programobject2dpu.trace2program() 
    #       print('(Step 210B/25) OF DPU2PROGRAM4XI5TEST - TRACE2DPU: ', trace2dpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4dpu-failed {}; time: 04:40:24 11/16/11 Pacific Standard Time}', 'CLASS': 'trace2program4dpu', 'METHOD': 'trace2program4dpu'}
    #   except:
    #       print('(Step 210Z1/25) OF DPU2PROGRAM4XI5TEST *** FAILED TO TRACE DPU: ', trace2dpu) 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2program(self, task4trace2program = None):
        '''
        traces DPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "trace2program"
                     
        message2print = "{(Step 1A/33) started" + "; trace2program-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2program-passed-to-trace-of-DPU2Program4Xi: " + str(task4trace2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2program-passing-to-trace2program4dpu-by-trace2program-of-DPU2Program4Xi: " + str(task4trace2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = trace2program4dpu(task4trace2program)

        message2print = "{(Step 2B/33) performed" + "; trace2program4dpu-performed-by-trace2program-of-DPU2Program4Xi: " + str(trace2program4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2program-of-DPU2Program4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "unload2compiler-of-DPU2Program4Xi" usage = "
    #        
    #   print('(Step 220A/25) OF DPU2PROGRAM4XI5TEST - TRYING TO UNLOAD COMPILER OF DPU: ', time4xi())      
    #   try:
    #       response4unload2compiler = programobject2dpu.unload2compiler() 
    #       print('(Step 220B/25) OF DPU2PROGRAM4XI5TEST - UNLOADED COMPILER OF DPU: ', response4unload2compiler)  
    #       # 
    #   except:
    #       print('(Step 220Z1/25) OF DPU2PROGRAM4XI5TEST *** FAILED TO TO UNLOAD COMPILER OF DPU: ', time4xi()) 
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def unload2compiler(self, task4unload2compiler = None):
        '''
        unloads compiler
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "unload2compiler"
                     
        message2print = "{(Step 1A/33) started" + "; unload2compiler-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4unload2compiler-passed-to-trace-of-DPU2Program4Xi: " + str(task4unload2compiler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4unload2compiler-passing-to-unload2compiler4dpu-by-unload2compiler-of-DPU2Program4Xi: " + str(task4unload2compiler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = unload2compiler4dpu(task4unload2compiler)

        message2print = "{(Step 2B/33) performed" + "; unload2compiler4dpu-performed-by-unload2compiler-of-DPU2Program4Xi: " + str(unload2compiler4dpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-unload2compiler-of-DPU2Program4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programcallback-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 200A/700) OF DPU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM CALLBACK WITH METHOD OF OBJECT XI2DPU: ', programobject2dpu) 
    #   try:
    #       callback2dpu = programobject2dpu.set2programcallback()                                  
    #       print('(Step 200B/700) OF DPU2PROGRAM4XI5TEST - SET PROGRAM WITH CALLBACK TO: ', callback2dpu)
    #       # <function trace2program4dpu at 0x00000000038E02C8>
    #   except:
    #       print('(Step 200Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM CALLBACK WITH METHOD ***', time4xi())
    #
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2programcallback(self, task4set2programcallback = None): 
        '''
        set2programcallback of DPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "DPU2Program4Xi"
        method2print = "set2programcallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2programcallback-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programcallback-passed-to-set2programcallback-of-DPU2Program4Xi: " + str(task4set2programcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
           
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programcallback-passing-to-set2programcallback-by-set2programcallback-of-DPU2Program4Xi: " + str(task4set2programcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        
        callback2dpu4program = {} 
        try:
            callback2dpu4program = set2programcallback4dpu (task4set2programcallback)

            
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2dpu4program-failed-by-set2programcallback-of-DPU2Program4Xi: " + str(callback2dpu4program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2dpu4program-returning-by-set2programcallback-of-DPU2Program4Xi: " + str(callback2dpu4program) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2program4dpu at 0x00000000038CF5C8> 
        
                                             
        return callback2dpu4program
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 310A/700) OF DPU2PROGRAM4XI5TEST - SENSING ', time4xi())  
    #   try:
    #       print(programobject2dpu.sense())
    #       print('(Step 310B/700) OF DPU2PROGRAM4XI5TEST - SENSED ', time4xi()) 
    #       # {'object': <dpu2program4xi.DPU2Program4Xi object at 0x0000000003E00AC8>, 'classvariable2dpu-of-DPU2Program4Xi': 4}
    #   except:
    #       print('(Step 310Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO SENSE ', time4xi())     
    #
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses DPU2Program4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'DPU2Program4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; DPU2Program4Xi-sensing-by-sense-of-DPU2Program4Xi: " + str(DPU2Program4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
            
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-DPU2Program4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2dpu-of-DPU2Program4Xi'] = DPU2Program4Xi.classvariable2dpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-DPU2Program4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; DPU2Program4Xi-sensed-by-sense-of-DPU2Program4Xi: " + str(DPU2Program4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-DPU2Program4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-DPU2Program4Xi" usage = "
    # 
    #   print('(Step 320A/700) OF DPU2PROGRAM4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(programobject2dpu.reason())
    #       print('(Step 320B/700) OF DPU2PROGRAM4XI5TEST - REASONED ', time4xi())
    #       # DPU2Program4Xi-reasoned-by-reason-of-DPU2Program4Xi: <class 'dpu2program4xi.DPU2Program4Xi'>; time: 04:40:25 11/16/11 Pacific Standard Time}
    #   except:
    #       print('(Step 320Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO REASON ', time4xi()) 
    #
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons DPU2Program4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'DPU2Program4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; DPU2Program4Xi-reasoning-by-reason-of-DPU2Program4Xi: " + str(DPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-DPU2Program4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
             
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-DPU2Program4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-DPU2Program4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; DPU2Program4Xi-reasoned-by-reason-of-DPU2Program4Xi: " + str(DPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-DPU2Program4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #           
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-DPU2Program4Xi" usage = "
    #
    #   print('(Step 330A/700) OF DPU2PROGRAM4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(programobject2dpu.ace())
    #       print('(Step 330B/700) OF DPU2PROGRAM4XI5TEST - ACED ', time4xi())                                                 
    #       # {'object-reason-object4xi': <dpu2program4xi.DPU2Program4Xi object at 0x0000000003E00AC8>, 'time-reason-of-DPU2Program4Xi': '04:40:25 11/16/11 Pacific Standard Time'}
    #   except:
    #       print('(Step 330Z/700) OF DPU2PROGRAM4XI5TEST *** FAILED TO ACE ', time4xi())                                 
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces DPU2Program4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'DPU2Program4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-DPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; DPU2Program4Xi-acing-by-ace-of-DPU2Program4Xi: " + str(DPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #      
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-DPU2Program4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-DPU2Program4Xi'] = DPU2Program4Xi.classvariable2dpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-DPU2Program4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; DPU2Program4Xi-aced-by-ace-of-DPU2Program4Xi: " + str(DPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'dpu2sourceprogram4xi.DPU2Program4Xi'> 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-DPU2Program4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>

# =================================================================================================
#<class4xi class = "DirectoryWalker" base = "None" child = "None" synopsis = "a forward iterator that traverses a directory tree" usage = "
#   to review all tests"
#>
# =================================================================================================
class DirectoryWalker:

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init" usage = "initializes DirectoryWalker"> 
    # ---------------------------------------------------------------------------------------------
    def __init__(self, directory):
        self.stack = [directory]
        self.files = []
        self.index = 0
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getitem" usage = "getting item"> 
    # ---------------------------------------------------------------------------------------------
    def __getitem__(self, index):
        while 1:
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------          
            try:
                # --------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------             
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # --------------------------------------------------------------------------------- 
                # pop next directory from stack
                # ---------------------------------------------------------------------------------             
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # --------------------------------------------------------------------------------- 
                # got a filename
                # ---------------------------------------------------------------------------------             
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                return fullname
    #</methodize4xi>
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   dpu2sourceprogram4xi5test.txt
#   sense-dpu2sourceprogram4xi5test.xml
#   reason-dpu2sourceprogram4xi5test.xml
#   ace-dpu2sourceprogram4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of dpu2program4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
